#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/3/27 10:56
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""

from christmas.read_conf import read_conf
from christmas import Blog
from christmas.commonCode import get_date, make_dir, osprint, split_path, osprints
import datetime
import itertools
from datetime import timedelta
import os


def select_file(_date, menu_path=r'Select_zip.conf', _logger=Blog().logger):
    menu = read_conf(menu_path)
    file_path = split_path(menu['SelectfromDir'])  # 原始文件的位置
    zip_path = split_path(menu['SelectputDir'])   # 压缩文件的位置
    dir_suffix = menu['DirSuffix']                                      # 压缩文件后缀名
    file_suffix = menu['FileSuffix']                                # 文件后缀名
    format_select = menu['Format_select']                         # 日期格式
    Format_date_py = menu['Format_date_py']                         # 日期格式
    Save_path = os.path.join(zip_path, f'{_date}{dir_suffix}')  # 保存路径
    # Zip_path = f'{Save_path}.zip'
    # path_to_del =[Save_path, Zip_path]
    # # 删除旧文件-------start
    # for i in path_to_del:
    #     if os.path.exists(i):
    #         os.system(f"rm -r {i}")
    #         osprint(f'已删除旧文件{i}！')
    make_dir(f'{Save_path}')
    # 删除旧文件并创建新文件夹-------end

    # 挑选和cp文件-------start
    folder_list = menu['Folder_list']
    filepre_list = menu['Filepre_list']
    time_list = menu['Switch_Select']

    for i, j in itertools.product(range(len(folder_list)), time_list):
        for day in range(int(menu['Select_duration'])):
            try:
                nd = datetime.datetime.strptime(_date, Format_date_py) + timedelta(day)
                input_path = f"{file_path}/{folder_list[i]}/{j}/{datetime.datetime.strftime(nd, '%Y%m%d')}"  # 插值文件输入位置
                output_path = f"{Save_path}/{folder_list[i]}/{j}/{datetime.datetime.strftime(nd, '%Y%m%d')}"
                make_dir(output_path)
                os.system(f"cp {input_path}/{filepre_list[i]}{file_suffix}{format_select} {output_path}")
            except Exception as e:
                continue
            if day == int(menu['Select_duration']) - 1:
                if j =='.':
                    _logger.info(f'{folder_list[i]} ---> {datetime.datetime.strftime(nd, "%Y%m%d")}')
                else:
                    _logger.info(f'{folder_list[i]}--{j} ---> {datetime.datetime.strftime(nd, "%Y%m%d")}')
    # 挑选和cp文件-------end


def zip_file(_date, menu_path=r'Select_zip.conf', _type='r', _logger=Blog().logger):
    menu = read_conf(menu_path)
    zip_path = split_path(menu['SelectputDir'])   # 压缩文件的位置
    dir_suffix = menu['DirSuffix']                                      # 压缩文件后缀名
    # 压缩
    pwd = os.getcwd()
    os.chdir(zip_path)
    if _type == 'r':  # 递归
        command = f"zip -qr {_date}{dir_suffix}.zip {_date}{dir_suffix}"
    elif _type == 'j':  # 不存储目录
        command = f"zip -qj {_date}{dir_suffix}.zip {_date}{dir_suffix}/*"
    else:
        raise ValueError('zip type error!')
    os.system(command)

    if not os.path.exists(f'{_date}{dir_suffix}.zip'):
        _logger.error(f'cd {zip_path} && {command} && cd {pwd}')
    else:
        _logger.info(f'已压缩{_date}{dir_suffix}.zip！')

    os.chdir(pwd)


def tgz_file(_date, menu_path=r'Select_zip.conf', _logger=Blog().logger):
    menu = read_conf(menu_path)
    zip_path = split_path(menu['SelectputDir'])   # 压缩文件的位置
    dir_suffix = menu['DirSuffix']                                      # 压缩文件后缀名
    # 压缩
    if os.path.exists(f'{_date}{dir_suffix}.tgz'):
        os.system(f"rm -r {_date}{dir_suffix}.tgz")
    pwd = os.getcwd()
    os.chdir(zip_path)
    os.system(f"tar czf {_date}{dir_suffix}.tgz {_date}{dir_suffix}")

    if not os.path.exists(f'{_date}{dir_suffix}.tgz'):
        _logger.error(f'tar czf {_date}{dir_suffix}.tgz {_date}{dir_suffix}')
    os.chdir(pwd)


def select_zip(_date, menu_path=r'Select_zip.conf'):
    select_file(_date, menu_path=menu_path)
    zip_file(_date, menu_path=menu_path)


if __name__ == '__main__':
    _date = get_date()
    select_zip(_date, menu_path='Select_zip.conf')



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/3/19 21:15
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
PythonAPI：https://help.marine.copernicus.eu/en/articles/8286883-copernicus-marine-toolbox-api-get-original-files
PythonOpendap: https://help.marine.copernicus.eu/en/articles/8287609-copernicus-marine-toolbox-api-open-a-dataset-or-read-a-dataframe-remotely
PythonCSV: https://help.marine.copernicus.eu/en/articles/7970637-how-to-download-data-for-multiple-points-from-a-csv
PythonRecurringperiod: https://help.marine.copernicus.eu/en/articles/7973161-how-to-get-data-for-a-recurring-period
"""



# -*- coding: utf-8 -*-
# @File    : lalala.py
# @Date    : 2023-04-11
# @Author  : Dovelet
import xarray as xarray
from metpy.interpolate import inverse_distance_to_grid
import numpy as np
import matplotlib.pyplot as plt
import time as T
from pykrige.ok import OrdinaryKriging
# 读取数据
plt.rcParams["font.family"] = ["sans-serif"]
plt.rcParams["font.sans-serif"] = ['SimHei']


# metpy cressman插值
def example1(radius=500/111000, neighbors=3):
    # metpy cressman插值
    # 通过改变r和min_neighbors可以改变插值的效果
    # r是欧氏距离，建议根据散点实际分辨率设置
    # min_neighbors是最小邻居数，如果找不到这么多邻居，就不插值
    f = xarray.open_dataset(r'ww3.20230417.nc')
    s_lon = f['longitude'].data
    s_lat = f['latitude'].data
    hs = f['hs'].data[0, :]

    lonmin = 120.15
    lonmax = 120.3
    latmin = 35.95
    latmax = 36.1
    re_ra = 400
    ##
    num_grid = int(round(latmax - latmin, 2) * re_ra)
    lon_new = np.linspace(lonmin, lonmax, num_grid)  # 创建插值格点
    lat_new = np.linspace(latmin, latmax, num_grid)

    lon_gridmesh, lat_gridmesh = np.meshgrid(lon_new, lat_new)

    start= T.time()
    tm_grid = inverse_distance_to_grid(s_lon, s_lat, hs, lon_gridmesh, lat_gridmesh, r=radius, min_neighbors=neighbors)
    end= T.time()
    print(f'metpy cressman插值耗时：{end-start}')
    plt.figure(1)
    plt.contourf(tm_grid)
    plt.colorbar()
    plt.title(f'metpy cressman插值 r={np.round(radius,2)},min_neighbors={neighbors}')
    plt.show()


# 克里金插值
def example2(model='linear'):
    # 一种对已知样本加权平均以估计平面上的未知点，并使得估计值与真实值的数学期望相同且方差最小的地统计学过程
    # 通过改变variogram_model可以改变插值的效果
    # pykrige提供 linear, power, gaussian, spherical, exponential, hole-effect几种variogram_model可供选择，默认的为linear模型
    f = xarray.open_dataset(r'ww3.20230417.nc')
    s_lon = f['longitude'].data
    s_lat = f['latitude'].data
    hs = f['hs'].data[0, :]

    lonmin = 120.15
    lonmax = 120.3
    latmin = 35.95
    latmax = 36.1
    re_ra = 400
    ##
    num_grid = int(round(latmax - latmin, 2) * re_ra)
    lon_new = np.linspace(lonmin, lonmax, num_grid)  # 创建插值格点
    lat_new = np.linspace(latmin, latmax, num_grid)

    # 需要对数据进行区域限定
    flag1=np.where((s_lon>=lonmin) & (s_lon<=lonmax))
    s_lon=s_lon[flag1]
    s_lat=s_lat[flag1]
    hs=hs[flag1]
    flag2=np.where((s_lat>=latmin) & (s_lat<=latmax))
    s_lon=s_lon[flag2]
    s_lat=s_lat[flag2]
    hs=hs[flag2]

    start = T.time()
    krige= OrdinaryKriging(s_lon, s_lat, hs, variogram_model=model, verbose=False, enable_plotting=False)
    tm_grid, ss = krige.execute('grid', lon_new, lat_new)
    end = T.time()
    print(f'克里金插值耗时：{end - start}')
    plt.figure(2)
    plt.contourf(tm_grid)
    plt.colorbar()
    plt.title(f'克里金线性插值，model：{model}')
    # 保存图片
    plt.show()
    plt.savefig(r'lalala2.png')


if __name__ == '__main__':
    example1()
    example2()



# -*- coding: utf-8 -*-

# @File    : Single_Forecast.py
# @Date    : 2022-09-22
# @Author  : 24672

import netCDF4
import numpy as np
import datetime
import os
import math
import scipy.integrate
import scipy.interpolate

import baysalt.strDateTime as strDateTime

m_hours = [key for key in range(216)]
''' [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
        24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45,
       46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67,
        68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93,
       94, 95, 96, 102, 108, 114, 120, 126, 132, 138, 144, 150, 156, 162, 168]'''


class Single_Forecast():

    def __init__(self,_lon,_lat,_input,_output,_name,_area):
        self._lon =_lon
        self._lat =_lat
        self._input=_input
        self._output=_output
        self._name=_name
        self._area=_area

    def getsp(self, _date):
        """
        索引ww3点位,找到最近的ww3散点
        :param _date: 任务日期
        :param _hour: 小时
        :return: 点位No
        """
        _date=str(_date)
        filename =self._input+_date+'/ww3.'+_date+'.nc'

        w = netCDF4.Dataset(filename)  # 读取ww3_rizhao的nc文件，传入w中

        lon = w.variables['longitude'][:]
        lat = w.variables['latitude'][:]
        D = np.zeros((lon.size, 1))
        for i in range(lon.size):
            D[i] = (self._lon - lon[i]) ** 2 + (self._lat - lat[i]) ** 2  # 读取ww3_rizhao的nc文件，传入w中
        p = np.argwhere(D == min(D))

        return p

    def getfvsp(self, _date):
        """
        索引fvcome点位,找到最近的fvcom散点
        :param _date: 任务日期
        :param _hour: 小时
        :return: 点位No
        """
        _date=str(_date)
        filename ='/data/Output_250/fvcom_EAMS/'+_date+'/EAMSforecast_0001.nc'
        w = netCDF4.Dataset(filename)   # 读取ww3_rizhao的nc文件，传入w中

        lon = w.variables['lonc'][:]
        lat = w.variables['latc'][:]
        D = np.zeros((lon.size, 1))
        for i in range(lon.size):
            D[i] = (self._lon - lon[i]) ** 2 + (self._lat - lat[i]) ** 2  # 读取ww3_rizhao的nc文件，传入w中
        fvp = np.argwhere(D == min(D))

        return fvp

    def getMaxHS(self, _hs):
        """
        根据有效波高计算最大波高
        :param _hs: 有效波高
        :return: 最大波高
        """
        hs_max = _hs * 1.46

        return hs_max

    def getWind(self, _date, _idx):
        """
        获取风
        :param _date:
        :param _idx:
        :return:风速，风向
        """
        # 时间处理
        date1 = datetime.datetime.strptime(_date + '00', '%Y%m%d%H')
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = '/home/ocean/Postprocess/160/Data/NC/wind10/'+self._area+'/'+date1.strftime('%Y%m%d')+'/'+self._area+'_'+date2.strftime(
            '%Y%m%d') + '.nc'   # 插值后的风场nc文件名
        # print(filename)
        nc_ds = netCDF4.Dataset(filename, 'r')

        lon = np.array(nc_ds.variables['longitude'])
        lat = np.array(nc_ds.variables['latitude'])
        latli = np.argmin(np.abs(lat - self._lat))
        lonli = np.argmin(np.abs(lon - self._lon))

        # 根据经纬度索引取值
        v = np.array(nc_ds.variables['v_VM'][idx, latli, lonli])
        dir = np.array(nc_ds.variables['dir_VM'][idx, latli, lonli])

        v_rtn = np.round(v, 2)
        dir_rtn = np.round(dir, 2)

        return str(v_rtn).rjust(14, " "), str(dir_rtn).rjust(14, " ")

    def getfvdata(self, _date, _idx, fvp, dep):
        """
        获取fvcome_eams数据
        :param _date: 任务日期
        :param _idx: 小时
        :param p: 点位
        :return: 流速，流向
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(_date + '00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = '/data/Output_250/fvcom_EAMS/' + date2.strftime(
            '%Y%m%d') + '/EAMSforecast_0001.nc'  # fvcom_EAMS nc文件名
        if not os.path.exists(filename):
            date3 = date2 + datetime.timedelta(days=-1)
            if not os.path.exists('/data/Output_250/fvcom_EAMS/' + date2.strftime('%Y%m%d')):
                os.mkdir('/data/Output_250/fvcom_EAMS/' + date2.strftime('%Y%m%d'))
            filename_old = '/data/Output_250/fvcom_EAMS/' + date3.strftime('%Y%m%d') + '/EAMSforecast_0001.nc'
            os.system('cp '+filename_old+' '+'/data/Output_250/fvcom_EAMS/' + date2.strftime('%Y%m%d'))
        w = netCDF4.Dataset(filename)   # 读取fvcom_EAMS的nc文件，传入w中
        fvcomu = w.variables['u'][idx, dep, fvp[0][0]]
        fvcomv = w.variables['v'][idx, dep, fvp[0][0]]

        # 计算流速和流向
        dir = math.atan2(fvcomu, fvcomv)
        dir = int(dir * 180 / math.pi)
        if dir < 0:
            dir = dir + 360
        speed = (fvcomu ** 2 + fvcomv ** 2) ** 0.5

        dir_rtn = np.round(float(dir), 2)
        speed_rtn = np.round(float(speed), 2)
        return str(speed_rtn).rjust(14, " "), str(dir_rtn).rjust(14, " ")

    def gettpxo(self, _date, _idx, dep):
        """
        获取tpxo数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 流速，流向
        """
        # 时间处理，不需要转cst
        date1 = datetime.datetime.strptime(_date + '00', '%Y%m%d%H')
        date2 = date1 + datetime.timedelta(hours=_idx)
        filename='/data/Postprocess/160/Data/input/tpxo/'+date2.strftime('%Y')+'_singlepoint_tpxo.nc'
        w = netCDF4.Dataset(filename, 'r', format='NETCDF4')
        lons = w.variables['longitude'][:]
        times = w.variables['TIME_CST'][:]

        row = 0
        for i in range(len(times)):
            if ("".join(times[i].astype(str).tolist())) == date2.strftime('%Y%m%d%H'):
                row = i
                break
        col = np.argmin(np.abs(lons - self._lon))

        # print(row,col)
        u_m = w.variables['u'][row, col]  # 中层流速u
        v_m = w.variables['v'][row, col]  # 中层流速v
        # 计算流速和流向
        dir = math.atan2(u_m, v_m)
        dir = int(dir * 180 / math.pi)
        if dir < 0:
            dir = dir + 360
        speed = (u_m ** 2 + v_m ** 2) ** 0.5

        if dep == 0:
            speed = 1.2 * speed  # 表层流速
        elif dep == 9:
            speed = 1.0 * speed  # 中层流速
        elif dep == 19:
            speed = 0.5 * speed  # 底层流速

        dir_rtn = np.round(float(dir), 2)
        speed_rtn = np.round(float(speed), 2)
        return str(speed_rtn).rjust(14, " "), str(dir_rtn).rjust(14, " ")

    def getDataFromSwan(self, _date, _idx, p):
        """
        获取ww3_djk数据
        :param _date: 任务日期
        :param _idx: 小时
        :param p: 点位
        :return: 有效波高，最大波高，涌浪有效波高，涌浪最大波高，风浪有效波高，风浪最大波高，波向，平均周期，风浪周期，涌浪周期，风浪波向，涌浪波向
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(_date + '00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
            '%Y%m%d') + '.nc'  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_djk的nc文件，传入w中
        ww3hs = w.variables['hs'][idx, :]  # 有效波高
        ww3t02 = w.variables['t02'][idx, :]   # 平均周期
        ww3max = w.variables['hmaxe'][idx, :]  # 最大波高
        ww3dir = w.variables['dir'][idx, :]   # 波向

        # 依靠最近点进行索引
        out_ww3hs = ww3hs[p[0][0]]  # 有效波高
        out_ww3t02 = ww3t02[p[0][0]]  # 波周期
        out_ww3max = ww3max[p[0][0]]  # 最大波高
        out_ww3dir = ww3dir[p[0][0]]  # 波向

        # print(out_ww3hs)

        # 1.海浪要素
        hs_rtn = np.round(float(out_ww3hs), 2)  # 有效波高
        t02_rtn = np.round(float(out_ww3t02), 2)  # 波周期
        hs_max = np.round(float(out_ww3max), 2)  # 最大波高
        out_ww3dir = np.where(out_ww3dir < 100.0, out_ww3dir + 50.0, out_ww3dir)
        out_ww3dir = np.where(out_ww3dir > 150.0, out_ww3dir - 30.0, out_ww3dir)
        dir_rtn = np.round(float(out_ww3dir), 2)  # 浪向

        # 读取一位谱数据用于下面计算风浪和涌浪
        ef_filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
            '%Y%m%d') + '_ef.nc'  # ww3 nc文件名
        ww3_ef = netCDF4.Dataset(ef_filename)
        ef = ww3_ef.variables['ef'][:, :, p[0][0]]
        f = ww3_ef.variables['f'][:]

        # 2.风浪要素
        # 新的风浪替代公式
        sf2 = ef[idx, 18:30]
        fs2 = f[18:30]
        wndhs = 4 * math.sqrt(scipy.integrate.trapz(sf2, fs2))  # 计算风浪有效波高
        wndtm = math.sqrt(scipy.integrate.trapz(sf2, fs2) / scipy.integrate.trapz(np.multiply(fs2 ** 2, sf2), fs2))  # 计算风浪有效周期
        wndmaxhs = self.getMaxHS(wndhs)  # 计算风浪最大波高

        wndhs_rtn = np.round(wndhs, 2)  # 风浪有效波高
        wndtm_rtn = np.round(wndtm, 2)  # 风浪周期
        wndmaxhs_rtn = np.round(wndmaxhs, 2)  # 风浪最大波高
        wnddir_rtn = dir_rtn  # 风浪波向用dir替代

        # 3.涌浪要素
        # 新的涌浪计算公式
        sf1 = ef[idx, 0:18]
        fs1 = f[0:18]
        shs = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))  # 计算涌浪有效波高
        swelltm =math.sqrt(scipy.integrate.trapz(sf1, fs1) / scipy.integrate.trapz(np.multiply(fs1 ** 2, sf1), fs1))  # 计算涌浪有效周期
        maxshs = self.getMaxHS(shs)  # 计算涌浪最大波高

        shs_rtn = np.round(shs, 2)  # 涌浪有效波高
        swelltm_rtn = np.round(swelltm, 2)  # 涌浪周期
        maxshs_rtn = np.round(maxshs, 2)  # 涌浪最大波高
        swelldir_rtn = dir_rtn # 涌浪波向dir替代

        if str(hs_max).strip() == "nan":  # 最大波高为nan，用1.46*HS
            hs_max = self.getMaxHS(hs_rtn)
            hs_max = np.round(float(hs_max), 2)  # 最大波高

        return str(hs_rtn).rjust(14, " "), str(hs_max).rjust(14, " "), str(shs_rtn).rjust(14, " "), \
               str(maxshs_rtn).rjust(14, " "), str(wndhs_rtn).rjust(14, " "), str(wndmaxhs_rtn).rjust(14, " "), \
               str(dir_rtn).rjust(14, " "), str(t02_rtn).rjust(14, " "), str(swelltm_rtn).rjust(14, " "), str(
            swelldir_rtn).rjust(14, " "), \
               str(wndtm_rtn).rjust(14, " "), str(wnddir_rtn).rjust(14, " ")

    def getH4H10CGE(self, _date, _idx, p):
        """
        获取ww3_djk数据
        :param _date: 任务日期
        :param _idx: 小时
        :param p: 点位
        :return: H4 H10 CGE PP
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(_date + '00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
            '%Y%m%d') + '.nc'  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_djk的nc文件，传入w中
        ww3hs = w.variables['hs'][idx, :]
        ww3cge = w.variables['cge'][idx, :]
        ww3fp = w.variables['fp'][idx, :]
        out_ww3hs = ww3hs[p[0][0]]  # 有效波高
        out_ww3cge = ww3cge[p[0][0]]  # 能量流
        out_ww3fp = ww3fp[p[0][0]]  # 谱峰周期

        # # 水深数据读取
        # py_file_path = os.path.dirname(os.path.abspath(__file__))  # 当前python文件所在位置
        # filename = py_file_path + '/depth_djk.nc'  # depth nc文件名
        # w = netCDF4.Dataset(filename)  # 读取水深的nc文件，传入w中
        # depth = w.variables['depth'][:]
        # d = depth[p[0][0]]
        # 平均波高Hp
        Hp = (0.747*out_ww3hs)/1.129
        # Hd = Hp/d

        # H4
        H4 = 2.024*Hp
        # H10
        H10 = 1.712*Hp
        # pp
        PP = 1/out_ww3fp

        H4_rtn = np.round(H4, 2)
        H10_rtn = np.round(H10, 2)
        cge_rtn = np.round(out_ww3cge, 2)
        PP_rtn = np.round(PP, 2)
        return str(H4_rtn).rjust(14, " "), str(H10_rtn).rjust(14, " "), str(cge_rtn).rjust(14, " "), str(PP_rtn).rjust(14, " ")

    def grtfl(self, _date):
        """
        构建预报单信息
        :param _date: 预报单日期
        :return: 0-生成预报数据成功  大于等于1-生成预报数据失败
        """
        isTemp = False  # 是否未来日期的扩展任务
        today = strDateTime.getToday()
        if _date <= today:
            isTemp = True
        else:
            isTemp = False

        lon_g =self._lon
        lat_g =self._lat
        outpath=self._output

        file_path =outpath
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        # 写入文件内容----------------------------------begin
        _date0 = _date
        list_hours = []
        list_hours.append(m_hours)

        sp = self.getsp(_date0)  # 索引点位
        # fvp = self.getfvsp(_date0)
        for k in range(0, 1):
            tmp = list_hours[k]
            list = []
            file_name = self._name + strDateTime.addDays(_date, k) + ".txt"
            file = os.path.join(file_path, file_name)

            fl = open(file, "w")
            # 写入文件头----------------------------------begin
            fh_time = " ".rjust(10, " ")  # 第一列日期留空白

            fh_hs = "swh".rjust(14, " ")  # 有效波高
            fh_dir = "mwd".rjust(14, " ")  # 波向
            fh_t01 = "mwp".rjust(14, " ")  # 平均周期
            fh_hmaxe = "hmax".rjust(14, " ")  # 最大波高

            fh_wwhs = "shww".rjust(14, " ")  # 风浪有效波高
            fh_wwdir = "mdww".rjust(14, " ")  # 风浪向
            fh_wwtm = "mpww".rjust(14, " ")  # 风浪平均周期
            fh_wwhmaxe = "hmaxw".rjust(14, " ")  # 风浪最大波高

            fh_swellhs = "shts".rjust(14, " ")  # 涌浪有效波高
            fh_swelldir = "mdts".rjust(14, " ")  # 涌浪向
            fh_swelltm = "mpts".rjust(14, " ")  # 涌浪平均周期
            fh_swellhmaxe = "hmaxs".rjust(14, " ")  # 涌浪最大波高

            fh_wndspeed = "windspeed".rjust(14, " ")  # 风速
            fh_wnddir = "winddir".rjust(14, " ")  # 风向

            fh_cspeeds = "currentspeeds".rjust(14, " ")  # 表层流速
            fh_cdirs = "currentdirs".rjust(14, " ")  # 表层流向
            fh_cspeedm = "currentspeedm".rjust(14, " ")  # 中层流速
            fh_cdirm = "currentdirm".rjust(14, " ")  # 中层流向
            fh_cspeedb = "currentspeedb".rjust(14, " ")  # 底层流速
            fh_cdirb = "currentdirb".rjust(14, " ")  # 底层流向

            fh_h4 = "h4".rjust(14, " ")  # 百分之4大波
            # fh_h10 = "H10".rjust(14, " ")  # 十分之1大波
            fh_CGE = "CGE".rjust(14, " ")  # 能量流
            fh_PP = "pp1d".rjust(14, " ")  # 谱峰周期

            fl.writelines([fh_time, fh_hs, fh_dir, fh_t01, fh_hmaxe, fh_wwhs, fh_wwdir, fh_wwtm, fh_wwhmaxe,
                           fh_swellhs, fh_swelldir, fh_swelltm, fh_swellhmaxe, fh_wndspeed, fh_wnddir,
                           fh_cspeeds, fh_cdirs, fh_cspeedm, fh_cdirm, fh_cspeedb, fh_cdirb,
                           fh_h4, fh_CGE, fh_PP, "\n"])
            # 写入文件头----------------------------------end

            for i in range(0, max(tmp) + 1):  # +72

                fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                fc_hs, fc_hmaxe, fc_swellhs, fc_swellhmaxe, fc_wwhs, fc_wwhmaxe, fc_dir, fc_t01,fc_swelltm, fc_swellDIR, fc_wwtm, fc_wwdir= \
                    self.getDataFromSwan(_date0, i, sp)  # 有效波高/最大波高

                fc_v, fc_wir = self.getWind(_date0, i)  # 风速/风向

                fc_h4, fc_h10, fc_CGE , fc_PP= self.getH4H10CGE(_date0, i, sp)  # H4/H10/CGE/PP

                fc_cspeeds, fh_cdirs = self.gettpxo(_date0, i, 0)
                fc_cspeedm, fh_cdirm = self.gettpxo(_date0, i, 9)
                fc_cspeedb, fh_cdirb = self.gettpxo(_date0, i, 19)
                # fc_cspeeds, fh_cdirs = self.getfvdata(_date0, i, fvp, 0)   #fvcom
                # fc_cspeedm, fh_cdirm = self.getfvdata(_date0, i, fvp, 9)
                # fc_cspeedb, fh_cdirb = self.getfvdata(_date0, i, fvp, 19)

                # if fc_wwdir.strip() == "nan" :  # 风浪方向为空时，采用综合波向代替
                #     fc_wwdir = fc_dir
                #
                # if fc_swellDIR.strip() == "nan":  # 涌浪方向为空时，采用综合波向代替
                #     fc_swellDIR = fc_dir
                #
                # if  fc_wwhs.strip() == "nan":  # 风浪浪高为空时，采用hs*0.7代替
                #     fc_wwhs=str(np.round(float(fc_hs) * 0.7, 2)).rjust(12, " ")
                #     fc_wwhmaxe=str(np.round(float(fc_wwhs) * 1.68, 2)).rjust(12, " ")

                if fc_PP.strip() == "--":  # 谱峰周期为空时，采用2.99代替
                    fc_PP = str(2.99).rjust(14, " ")
                if (i in m_hours):
                    fl.writelines([fc_time, fc_hs, fc_dir, fc_t01, fc_hmaxe, fc_wwhs, fc_wwdir, fc_wwtm, fc_wwhmaxe,
                                   fc_swellhs, fc_swellDIR, fc_swelltm, fc_swellhmaxe, fc_v, fc_wir,
                                   fc_cspeeds, fh_cdirs, fc_cspeedm, fh_cdirm, fc_cspeedb, fh_cdirb,
                                   fc_h4, fc_CGE, fc_PP, "\n"])
                    print("hour in=" + str(i))
                else:
                    print("hour=" + str(i))

            fl.close()

    def getTime(self, _date, _hour):
        """
        获取预报的日期/小时
        :param _date: 任务日期
        :param _hour: 小时
        :return: 日期/小时(10位长度的字符串)
        """
        if _hour == 215:
            _hour = 215

        date = strDateTime.convertToTime(_date)
        date_t = date + datetime.timedelta(hours=int(_hour))

        return date_t.strftime('%Y%m%d%H')








# -*- coding: utf-8 -*-

# @File    : newnew_cangnan.py
# @Date    : 2023-07-26
# @Author  : Dovelet
# -*- coding: utf-8 -*-

"""
1.增加流速提取的部分
2.将WW3里识别为陆地的地方采取idw插值的方式进行填充
"""

import netCDF4
import numpy as np
import os
import datetime
import baysalt.strDateTime as strDateTime
from scipy.interpolate import interp2d
from scipy.interpolate import interpn
import scipy.integrate
import json
import math
import scipy.spatial as spt
import itertools

m_hours = list(range(120))
fvcom_dir='/data/Output_147/FVCOM_ECS_storm/Run/'
tpxo_dir='/home/ocean/Project/Cangnan_hedian/Data/input/tpxo/'

class Cangnan_Single:
    def __init__(self, _lon, _lat, _input, _output_txt, _output_json, _name, **_new_dict):
        self._lon = _lon
        self._lat = _lat
        self._input = _input
        self._output_txt = _output_txt
        self._output_json = _output_json
        self._name = _name
        self._new_dict = _new_dict
        self.p=None

    def getnum(self, _date, _idx,para1,para2):
        """
        获取ww3_djk数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
            '%Y%m%d') + '.nc'  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_Wp的nc文件，传入w中
        lons = w.variables['longitude'][:]
        lats = w.variables['latitude'][:]
        ww3hs = w.variables['hs'][idx, :, :]  # 有效波高
        ww3t02 = w.variables['t02'][idx, :, :]  # 平均周期
        ww3fp = w.variables['fp'][idx, :, :]  # 谱峰频率
        ww3dir = w.variables['dir'][idx, :, :]  # 波向
        ww3swellhs = w.variables['phs1'][idx, :, :]  # 涌浪有效波高
        ww3swelltm = w.variables['ptp1'][idx, :, :]  # 涌浪谱峰周期
        ww3lm = w.variables['lm'][idx, :, :]  # 波长

        fhs = interp2d(lons, lats, ww3hs, kind='linear')
        hs = fhs(self._lon, self._lat)
        if hs[0] < 1e+10:
            ft02 = interp2d(lons, lats, ww3t02, kind='linear')
            ffp = interp2d(lons, lats, ww3fp, kind='linear')
            fdir = interp2d(lons, lats, ww3dir, kind='linear')
            fswellhs = interp2d(lons, lats, ww3swellhs, kind='linear')
            fswelltm = interp2d(lons, lats, ww3swelltm, kind='linear')
            flm = interp2d(lons, lats, ww3lm, kind='linear')
            # H4
            t02 = ft02(self._lon, self._lat)
            # H10
            mwd = fdir(self._lon, self._lat)
            # pp
            PP = 1 / ffp(self._lon, self._lat)
            # phs1
            phs1 = fswellhs(self._lon, self._lat)
            # ptp1
            ptp1 = fswelltm(self._lon, self._lat)
            # LM
            lm = flm(self._lon, self._lat)
            if phs1[0] >= 1e+10 or ptp1[0] >= 1e+10:  # 涌浪波高或谱峰周期为空时用一维谱计算
                ef_filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
                    '%Y%m%d') + '_ef.nc'  # ww3_ef nc文件名
                ww3_ef = netCDF4.Dataset(ef_filename)
                ww3ef = ww3_ef.variables['ef'][idx, :, :, :]
                f = ww3_ef.variables['f'][:]

                Z, Y, X = np.meshgrid(f, self._lat, self._lon, indexing='ij')
                ef = interpn((f, lats, lons), ww3ef, (Z, Y, X), method='linear')  # 维度和值的维度是正的

                # 涌浪要素
                # 新的涌浪计算公式
                sf1 = ef[:18, 0, 0]
                fs1 = f[:18]
                phs1[0] = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))  # 计算涌浪有效波高
                ptp1[0] = math.sqrt(
                    scipy.integrate.trapz(sf1, fs1) / scipy.integrate.trapz(np.multiply(fs1 ** 2, sf1),fs1))  # 计算涌浪有效周期
            # hs_rtn = np.round(hs[0], 2)
            # t02_rtn = np.round(t02[0], 2)
            # dir_rtn = np.round(mwd[0], 2)
            # PP_rtn = np.round(PP[0], 2)
            # phs1_rtn = np.round(phs1[0], 2)
            # ptp1_rtn = np.round(ptp1[0], 2)
            # lm_rtn = np.round(lm[0], 2)
        else:
            hs =self.gts_idw(ww3hs, para1, para2)
            t02 = self.gts_idw(ww3t02, para1, para2)
            mwd = self.gts_idw(ww3dir, para1, para2)
            PP = 1 / self.gts_idw(ww3fp, para1, para2)
            phs1 = self.gts_idw(ww3swellhs, para1, para2)
            ptp1 = self.gts_idw(ww3swelltm, para1, para2)
            lm = self.gts_idw(ww3lm, para1, para2)
            
            if str(phs1) =='--' or str(ptp1)=='--':  # 涌浪波高或谱峰周期为空时用一维谱计算
                ef_filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
                    '%Y%m%d') + '_ef.nc'  # ww3_ef nc文件名
                ww3_ef = netCDF4.Dataset(ef_filename)
                ww3ef = ww3_ef.variables['ef'][idx, :, :, :]
                f = ww3_ef.variables['f'][:]

                Z, Y, X = np.meshgrid(f, self._lat, self._lon, indexing='ij')
                ef = interpn((f, lats, lons), ww3ef, (Z, Y, X), method='linear')  # 维度和值的维度是正的

                # 涌浪要素
                # 新的涌浪计算公式
                sf1 = ef[:18, 0, 0]
                fs1 = f[:18]
                phs1 = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))  # 计算涌浪有效波高
                ptp1 = math.sqrt(
                    scipy.integrate.trapz(sf1, fs1) / scipy.integrate.trapz(np.multiply(fs1 ** 2, sf1),fs1))  # 计算涌浪有效周期

            if phs1 >= 1e+10 or ptp1 >= 1e+10:
               phs1=self.judge_swell(ww3swellhs,para1)
               ptp1=self.judge_swell(ww3swelltm,para1)

            if not phs1 or not ptp1:
                phs1=np.nan
                ptp1=np.nan

        hs_rtn = np.round(np.float64(hs), 2)
        t02_rtn = np.round(np.float64(t02), 2)
        dir_rtn = np.round(np.float64(mwd), 2)
        PP_rtn = np.round(np.float64(PP), 2)
        phs1_rtn = np.round(np.float64(phs1), 2)
        ptp1_rtn = np.round(np.float64(ptp1), 2)
        lm_rtn = np.round(np.float64(lm), 2)

        return hs_rtn, t02_rtn, dir_rtn, PP_rtn, phs1_rtn, ptp1_rtn, lm_rtn


    def judge_swell(self, value,para1):
        for i in range(len(para1)):
            if str(value[para1[i]])=='--':
                continue
            else:
                return value[para1[i]]




    def getfore(self, _date, _idx,para1,para2):
        """
        获取ww3_djk数据,流场数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向
        """
        hs_rtn, t02_rtn, dir_rtn, PP_rtn, phs1_rtn, ptp1_rtn, lm_rtn = self.getnum(_date, _idx,para1,para2)
        
        uvh_avg = self.getcurrent(_date, _idx,self.p)
        fc_cmu = uvh_avg['ua']
        fc_cmv = uvh_avg['va']
        fc_h = uvh_avg['zeta']

        return str(format(hs_rtn, '.2f')).rjust(12, " "), \
               str(format(t02_rtn, '.2f')).rjust(12, " "), \
               str(format(dir_rtn, '.2f')).rjust(12, " "), \
               str(format(PP_rtn, '.2f')).rjust(12, " "), \
               str(format(phs1_rtn, '.2f')).rjust(12, " "), \
               str(format(ptp1_rtn, '.2f')).rjust(12, " "), \
               str(format(lm_rtn, '.2f')).rjust(12, " "), \
                str(format(fc_cmu, '.2f')).rjust(12, " "), \
                str(format(fc_cmv, '.2f')).rjust(12, " "), \
                str(format(fc_h, '.2f')).rjust(12, " ")




    def getcurrent(self,_date,_idx,p):
        """
        获取fvcom数据
        :param _date: 日期
        :param _idx: 预报小时（需要做时间处理）
        :param p: fvcom网格点的索引
        :return: 流uv，深度
        """
        # 时间处理，将UTC转为CST

        if _idx>=120:
            uvh_avg = {'ua': np.nan, 'va': np.nan, 'zeta': np.nan}
            return uvh_avg
        _idx_tpxo = _idx
        _date_tpxo = _date
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H') # 预报起始时间
        date3 = date1 + datetime.timedelta(days=-1)  # 预报起始前一天
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx) # 预报时刻
        idx = _idx % 24

        if _idx<0:
            filename=os.path.join(fvcom_dir,date2.strftime('%Y%m%d')+'/output'+'/forecast_0001.nc')

        else:

            # 一个小天才的写法，哈哈哈哈
            filename=os.path.join(fvcom_dir,date2.strftime('%Y%m%d')+'/output'+'/forecast_0001.nc')

            if os.path.exists(filename)==False:

                filename=os.path.join(fvcom_dir,date1.strftime('%Y%m%d')+'/output'+'/forecast_'+str(int(_idx/24)+1).zfill(4)+'.nc')

            if os.path.exists(filename)==False:

                filename=os.path.join(fvcom_dir,date3.strftime('%Y%m%d')+'/output'+'/forecast_'+str(int(_idx/24)+2).zfill(4)+'.nc')

        f_c = netCDF4.Dataset(filename) #fvcom nc
        ua = f_c.variables['ua'][:]
        va = f_c.variables['va'][:]
        # 环流的u,v
        ua_model = ua[idx,  p[0]]
        va_model = va[idx,  p[0]]
        # u_s= u[idx,0, p[0]]
        # v_s= v[idx,0, p[0]]
        # u_m= u[idx,1, p[0]]
        # v_m= v[idx,1, p[0]]
        # u_b= u[idx,2, p[0]]
        # v_b= v[idx,2, p[0]]
        # 水位
        h_zeta = f_c.variables['zeta'][:]
        zeta=h_zeta[idx,p[1]]

        # 潮流的u,v
        uvh_tpxo_dict = self.get_tpxo(_date_tpxo, _idx_tpxo)
        # tpxo_us = uv_tpxo_dict['u_s']
        # tpxo_vs = uv_tpxo_dict['v_s']
        # tpxo_um = uv_tpxo_dict['u_m']
        # tpxo_vm = uv_tpxo_dict['v_m']
        # tpxo_ub = uv_tpxo_dict['u_b']
        # tpxo_vb = uv_tpxo_dict['v_b']
        tpxo_u_mean = uvh_tpxo_dict['u_mean']
        tpxo_v_mean = uvh_tpxo_dict['v_mean']
        tpxo_h_mean = uvh_tpxo_dict['h']

        # 合并环流和潮流的UVh
        ua = ua_model + tpxo_u_mean
        va = va_model + tpxo_v_mean
        zeta = zeta + tpxo_h_mean
        
        # u_s = u_s + tpxo_us
        # v_s = v_s + tpxo_vs
        # u_m = u_m + tpxo_um
        # v_m = v_m + tpxo_vm
        # u_b = u_b + tpxo_ub
        # v_b = v_b + tpxo_vb
        # 保留两位小数
        ua = np.round(np.float64(ua), 2)
        va = np.round(np.float64(va), 2)
        # u_s = np.round(np.float64(u_s), 2)
        # v_s = np.round(np.float64(v_s), 2)
        # u_m = np.round(np.float64(u_m), 2)
        # v_m = np.round(np.float64(v_m), 2)
        # u_b = np.round(np.float64(u_b), 2)
        # v_b = np.round(np.float64(v_b), 2)

        zeta=np.round(np.float64(zeta),2)
        
        uvh_avg = {'ua': ua, 'va': va, 'zeta': zeta}

        return uvh_avg


    def get_current_sp(self,_date):
        """
        找到最近的fvcom数据点的索引
        :param _date: 日期。
        :return: 点位索引
        """
        _date = str(_date)
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')  # 预报起始时间
        date3 = date1 + datetime.timedelta(days=-1)  # 预报起始前一天
        date3= date3.strftime('%Y%m%d')
        filename = os.path.join(fvcom_dir, date3 + '/output' + '/forecast_0001.nc')
        f_c = netCDF4.Dataset(filename)  # fvcom nc
        lonc= f_c.variables['lonc'][:]
        latc= f_c.variables['latc'][:]
        D=np.sqrt((lonc-self._lon)**2+(latc-self._lat)**2)
        idx_uv=np.argmin(D)

        lon=f_c.variables['lon'][:]
        lat=f_c.variables['lat'][:]
        D=np.sqrt((lon-self._lon)**2+(lat-self._lat)**2)
        idx_h=np.argmin(D)

        return [idx_uv,idx_h]


    def get_tpxo(self,_date,_idx):
        date1 = datetime.datetime.strptime(_date + '00', '%Y%m%d%H')
        date2 = date1 + datetime.timedelta(hours=_idx)
        date2 = date2.strftime('%Y%m%d%H')
        filename= os.path.join(tpxo_dir, date2[0:4]+'_singlepoint_tpxo.nc')
        f_t = netCDF4.Dataset(filename)  # tpxo nc
        lons = f_t.variables['longitude'][:]

        col = np.argmin(np.abs(lons - self._lon))
        row = self.hourInyear(date2)
        # 潮流的u,v
        u_m = f_t.variables['u'][row, col]  # 中层流速u
        v_m = f_t.variables['v'][row, col]  # 中层流速v
        h = f_t.variables['h'][row, col]  # 潮位
        # 计算表层流速和底层
        u_s = u_m*np.sqrt(1.2)
        v_s = v_m*np.sqrt(1.2)
        u_b = u_m*np.sqrt(0.5)
        v_b = v_m*np.sqrt(0.5)
        
        u_mean = u_m
        v_mean = v_m
        
        uvh = {'u_s': u_s, 'v_s': v_s, 'u_m': u_m, 'v_m': v_m, 'u_b': u_b, 'v_b': v_b, 'u_mean': u_mean, 'v_mean': v_mean,'h':h}

        return uvh


    def hourInyear(self,given_time):
        given_time = datetime.datetime.strptime(given_time, "%Y%m%d%H")
        day_of_year = given_time.timetuple().tm_yday  # 获取该时刻是一年中的第几天
        hour_of_day = given_time.hour  # 获取该时刻是当天的第几个小时
        return (day_of_year - 1) * 24 + hour_of_day

    @staticmethod
    def getTime(_date, _hour):
        """
        获取预报的日期/小时
        :param _date: 任务日期
        :param _hour: 小时
        :return: 日期/小时(10位长度的字符串)
        """
        if _hour == 215:
            _hour = 215

        date = strDateTime.convertToTime(_date)
        date_t = date + datetime.timedelta(hours=int(_hour))

        return date_t.strftime('%Y%m%d%H')

    @staticmethod
    def nan2str(_data):
        # 将nan转换为字符串
        if np.isnan(_data):
            return 'NaN'
        else:
            return _data

    @staticmethod
    def getJsonData(date, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1, fc_cmu, fc_cmv, fc_h):
        # sourcery skip: dict-literal, inline-immediately-returned-variable, merge-dict-assign
        postData = dict()
        postData['date'] = date
        postData['swh'] = fc_hs
        postData['mwd'] = fc_dir
        postData['mwp'] = fc_t01
        postData['pp1d'] = Cangnan_Single.nan2str(fc_PP)
        postData['shts'] = Cangnan_Single.nan2str(fc_phs1)
        postData['mpts'] = Cangnan_Single.nan2str(fc_ptp1)
        postData['mwl'] = Cangnan_Single.nan2str(fc_lm)
        postData['u_average'] = Cangnan_Single.nan2str(fc_cmu)
        postData['v_average'] = Cangnan_Single.nan2str(fc_cmv)
        # postData['u_surface'] = Cangnan_Single.nan2str(fc_csu)
        # postData['v_surface'] = Cangnan_Single.nan2str(fc_csv)
        # postData['u_middle'] = Cangnan_Single.nan2str(fc_cmu)
        # postData['v_middle'] = Cangnan_Single.nan2str(fc_cmv)
        # postData['u_bottom'] = Cangnan_Single.nan2str(fc_cbu)
        # postData['v_bottom'] = Cangnan_Single.nan2str(fc_cbv)
        postData['h'] = Cangnan_Single.nan2str(fc_h)
        return postData

    def grttxt(self, _date):
        """
        构建预报单信息
        :param _date: 预报单日期
        :return: 0-生成预报数据成功  大于等于1-生成预报数据失败
        """

        today = strDateTime.getToday()
        file_path = self._output_txt
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        # 写入文件内容----------------------------------begin
        _date0 = _date
        list_hours = [m_hours]
        # 写入文件头----------------------------------begin
        fh_time = "time".rjust(12, " ")  # 第一列日期留空白
        fh_hs = "swh".rjust(12, " ")  # 有效波高
        fh_dir = "mwd".rjust(12, " ")  # 波向
        fh_t01 = "mwp".rjust(12, " ")  # 平均周期
        fh_lm = 'mwl'.rjust(12, " ")  # 波长
        fh_PP = "pp1d".rjust(12, " ")  # 谱峰周期
        fh_phs1 = 'shts'.rjust(12, " ")  # 涌浪有效波高
        fh_ptp1 = 'mpts'.rjust(12, " ")  # 涌浪谱峰周期
        # 流场数据开头
        fh_cmu='u_average'.rjust(12, " ")
        fh_cmv='v_average'.rjust(12, " ")
        # fh_css='u_surface'.rjust(12, " ")
        # fh_csm='v_surface'.rjust(12, " ")
        # fh_csb='u_middle'.rjust(12, " ")
        # fh_cds='v_middle'.rjust(12, " ")
        # fh_cdm='u_bottom'.rjust(12, " ")
        # fh_cdb='v_bottom'.rjust(12, " ")
        fh_h='h'.rjust(12, " ")

        self.p = self.get_current_sp(_date)
        para1, para2 = self.judge_method(_date)  # 反距离加权插值的参数


        for k in range(1):
            tmp = list_hours[k]
            file_name = self._name + strDateTime.addDays(_date, k) + ".txt"
            file = os.path.join(file_path, file_name)

            with open(file, "w") as fl:
                fl.writelines([fh_time, fh_hs, fh_dir, fh_t01, fh_lm, fh_PP, fh_phs1, fh_ptp1,fh_cmu,fh_cmv,fh_h,"\n"])
                # 写入文件头----------------------------------end

                for i in range(max(tmp) + 1):  # +72

                    fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                    fc_hs, fc_t01, fc_dir, fc_PP, fc_phs1, fc_ptp1, fc_lm, fc_cmu, fc_cmv, fc_h = self.getfore(_date0, i,para1,para2)  # 获得数据

                    # if fc_PP.strip() == "nan":  # 谱峰周期为空时，采用2.99代替
                    #     fc_PP = str(2.99).rjust(12, " ")
                    if i in m_hours:

                        fl.writelines([fc_time, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1, fc_cmu, fc_cmv, fc_h,"\n"])
                        print(f"hour in={str(i)}")
                    else:
                        print(f"hour={str(i)}")

    def grtjson(self, _date):

        file_path = self._output_json
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        _date0 = _date
        list_hours = [m_hours]
        self.p = self.get_current_sp(_date)
        para1, para2 = self.judge_method(_date)  # 反距离加权插值的参数
        # 创建json数据----------------------------------begin
        for k in range(1):
            tmp = list_hours[k]
            list = []
            for i in range(max(tmp) + 1):

                fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                fc_hs, fc_t01, fc_dir, fc_PP, fc_phs1, fc_ptp1, fc_lm = self.getnum(_date0, i,para1, para2)  # 获取数据
                uvh_avg = self.getcurrent(_date0, i,self.p)
                fc_cmu = uvh_avg['ua']
                fc_cmv = uvh_avg['va']
                fc_h = uvh_avg['zeta']

                # if np.isnan(fc_PP):  # 谱峰周期为空时，采用2.99代替
                #     fc_PP = 2.99

                JsonData = self.getJsonData(fc_time, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1, fc_cmu, fc_cmv, fc_h)  # dict()
                list.append(JsonData)
                # 创建json数据----------------------------------end
                # 创建json文件----------------------------------begin
                file_name = self._name + strDateTime.addDays(_date, k) + ".json"
                file = os.path.join(file_path, file_name)
                # new_dict = {"location": {"No": "rzlssh01", "name": "日照岚山港区原油码头", "lon": lon_g, "lat": lat_g}, "data": list}
                new_dict = {"location": self._new_dict, "data": list}
                with open(file, "w") as f:
                    json.dump(new_dict, f, ensure_ascii=False, indent=10)
                # 写入文件内容----------------------------------end

    def judge_method(self, _date):

        _date1 = datetime.datetime.strptime(f'{_date}', '%Y%m%d')
        filename = self._input + _date1.strftime('%Y%m%d') + '/ww3.' + _date1.strftime(
            '%Y%m%d') + '.nc'  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_Wp的nc文件，传入w中
        lons = w.variables['longitude'][:]
        lats = w.variables['latitude'][:]
        value = w.variables['hs'][0, :, :]  # 有效波高
        method = 'idw'
        cal_id, cal_pa = self.idw_para(lons, lats, value, self._lon, self._lat,12)

        return cal_id, cal_pa

    @staticmethod
    def gts_idw(value, cal_id, cal_pa):
        result = 0
        for i in range(len(cal_id)):
            result = result + cal_pa[i] * value[cal_id[i]]
        return result

    def idw_para(self, g_lon, g_lat, value, s_lon, s_lat,search_point):
        lat_number = len(g_lat)
        lon_number = len(g_lon)
        # 生成格点坐标，放进一个二维平面
        grid = []
        for i, j in itertools.product(range(lon_number), range(lat_number)):
            x = [g_lon[i], g_lat[j]]
            grid.append(x)
        point = np.array(grid)

        # 用于快速查找的KDTree类
        kt = spt.KDTree(data=point, leafsize=10)
        ckt = spt.cKDTree(point)  # 用C写的查找类，执行速度更快
        find_point = np.array([s_lon, s_lat])  # 原点
        distance, sequence = kt.query(find_point, search_point)  # 返回最近邻点的距离d和在数组中的顺序sequence
        fp = []  # 存格点
        if search_point == 1:
            fp.extend(point[i] for i in range(len(point)) if i == sequence)
        else:
            fp.extend(point[p] for p in sequence)
        index = []  # 存索引
        for f in fp:
            id = self.find_index(g_lon, g_lat, f[0], f[1])
            index.append(id)

        cal_id = []  # 存可以进入计算的点的索引
        cal_d = []
        for i in range(len(index)):
            if self.detect_nan(value[index[i]]):
                cal_id.append(index[i])
                cal_d.append(distance[i])
        # 反距离加权参数的计算
        cal_pa = []
        total = 0
        for item in cal_d:
            total = total + 1 / item
        cal_pa.extend((1 / dis) / total for dis in cal_d)
        return cal_id, cal_pa


    @staticmethod
    def detect_nan(value):
        """
        判断格点数据是否满足双线性插值条件
        :param value: 取出的格点数据
        :return: 满足则返回1，不满足则返回0
        """
        try:
            if value.mask:
                return 0
        except Exception:
            return 1

    @staticmethod
    def find_index(g_lon, g_lat, s_lon, s_lat):
        """
        获取经纬度索引
        :param g_lon: 一维格点经度
        :param g_lat: 一维格点纬度
        :param s_lon: 散点经度
        :param s_lat: 散点纬度
        :return: 纬度索引 经度索引
        """
        latli = np.argmin(np.abs(g_lat - s_lat))
        lonli = np.argmin(np.abs(g_lon - s_lon))
        return latli, lonli


if __name__ == '__main__':
    input_path = '/data/ForecastSystem/WW3_6.07/Output/ww3_WPacific/'
    py_file_path = os.path.dirname(os.path.abspath(__file__))
    output_path_txt = os.path.join(py_file_path, './Data/single_txt/')
    output_path_json = os.path.join(py_file_path, './Data/single_json/')
    if not os.path.exists(output_path_txt):
        os.makedirs(output_path_txt)
    if not os.path.exists(output_path_json):
        os.makedirs(output_path_json)
    location_xdhy1 = {"No": "bdhy1", "name": "浅滩北堤海域1", "lon": 121.071017,
                      "lat": 27.926412}
    name_xdhy1 = "bdhy1_121.071E_27.926N_"
    c=Cangnan_Single( 121.071017, 27.926412, input_path, output_path_txt, output_path_json, name_xdhy1,**location_xdhy1)

    #c.grttxt('20230728')
    print('json')
    c.grtjson('20230728')
    print('end')



# -*- coding: utf-8 -*-

# @File    : new_single.py
# @Date    : 2022-12-30
# @Author  : Dovelet
import netCDF4
import numpy as np
import os
import datetime
import baysalt.strDateTime as strDateTime
from scipy.interpolate import interp2d
from scipy.interpolate import interpn
import scipy.integrate
import json
import math

m_hours = list(range(216))


class Cangnan_Single:
    def __init__(self, _lon, _lat, _input, _output_txt, _output_json, _name, **_new_dict):
        self._lon = _lon
        self._lat = _lat
        self._input = _input
        self._output_txt = _output_txt
        self._output_json = _output_json
        self._name = _name
        self._new_dict = _new_dict
    
    def getnum(self, _date, _idx):
        """
        获取ww3_djk数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
            '%Y%m%d') + '.nc'  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_Wp的nc文件，传入w中
        lons = w.variables['longitude'][:]
        lats = w.variables['latitude'][:]
        ww3hs = w.variables['hs'][idx, :, :]  # 有效波高
        ww3t02 = w.variables['t02'][idx, :, :]  # 平均周期
        ww3fp = w.variables['fp'][idx, :, :]  # 谱峰频率
        ww3dir = w.variables['dir'][idx, :, :]  # 波向
        ww3swellhs = w.variables['phs1'][idx, :, :]  # 涌浪有效波高
        ww3swelltm = w.variables['ptp1'][idx, :, :]  # 涌浪谱峰周期
        ww3lm = w.variables['lm'][idx, :, :]  # 波长

        fhs = interp2d(lons, lats, ww3hs, kind='linear')
        ft02 = interp2d(lons, lats, ww3t02, kind='linear')
        ffp = interp2d(lons, lats, ww3fp, kind='linear')
        fdir = interp2d(lons, lats, ww3dir, kind='linear')
        fswellhs = interp2d(lons, lats, ww3swellhs, kind='linear')
        fswelltm = interp2d(lons, lats, ww3swelltm, kind='linear')
        flm = interp2d(lons, lats, ww3lm, kind='linear')

        hs = fhs(self._lon, self._lat)
        # H4
        t02 = ft02(self._lon, self._lat)
        # H10
        mwd = fdir(self._lon, self._lat)
        # pp
        PP = 1 / ffp(self._lon, self._lat)
        # phs1
        phs1 = fswellhs(self._lon, self._lat)
        # ptp1
        ptp1 = fswelltm(self._lon, self._lat)
        # LM
        lm = flm(self._lon, self._lat)

        if phs1[0] >= 1e+10 or ptp1[0] >= 1e+10:  # 涌浪波高或谱峰周期为空时用一维谱计算
            ef_filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
                '%Y%m%d') + '_ef.nc'  # ww3_ef nc文件名
            ww3_ef = netCDF4.Dataset(ef_filename)
            ww3ef = ww3_ef.variables['ef'][idx, :, :, :]
            f = ww3_ef.variables['f'][:]

            Z, Y, X = np.meshgrid(f, self._lat, self._lon, indexing='ij')
            ef = interpn((f, lats, lons), ww3ef, (Z, Y, X), method='linear')  # 维度和值的维度是正的

            # 涌浪要素
            # 新的涌浪计算公式
            sf1 = ef[:18, 0, 0]
            fs1 = f[:18]
            phs1[0] = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))  # 计算涌浪有效波高
            ptp1[0] = math.sqrt(
                scipy.integrate.trapz(sf1, fs1) / scipy.integrate.trapz(np.multiply(fs1 ** 2, sf1), fs1))  # 计算涌浪有效周期

        hs_rtn = np.round(hs[0], 2)
        t02_rtn = np.round(t02[0], 2)
        dir_rtn = np.round(mwd[0], 2)
        PP_rtn = np.round(PP[0], 2)
        phs1_rtn = np.round(phs1[0], 2)
        ptp1_rtn = np.round(ptp1[0], 2)
        lm_rtn = np.round(lm[0], 2)

        return hs_rtn, t02_rtn, dir_rtn, PP_rtn, phs1_rtn, ptp1_rtn, lm_rtn
    
    def getfore(self, _date, _idx):
        """
        获取ww3_djk数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向
        """
        hs_rtn, t02_rtn, dir_rtn, PP_rtn, phs1_rtn, ptp1_rtn, lm_rtn = self.getnum(_date, _idx)
        return str(format(hs_rtn, '.2f')).rjust(10, " "), \
            str(format(t02_rtn, '.2f')).rjust(10, " "), \
            str(format(dir_rtn, '.2f')).rjust(10, " "), \
            str(format(PP_rtn, '.2f')).rjust(10, " "), \
            str(format(phs1_rtn, '.2f')).rjust(10, " "),\
            str(format(ptp1_rtn, '.2f')).rjust(10, " "),\
            str(format(lm_rtn, '.2f')).rjust(10, " ")
    
    @staticmethod
    def getTime(_date, _hour):
        """
        获取预报的日期/小时
        :param _date: 任务日期
        :param _hour: 小时
        :return: 日期/小时(10位长度的字符串)
        """
        if _hour == 215:
            _hour = 215
        
        date = strDateTime.convertToTime(_date)
        date_t = date + datetime.timedelta(hours=int(_hour))
        
        return date_t.strftime('%Y%m%d%H')
    
    @staticmethod
    def getJsonData(date, fc_hs, fc_dir, fc_t01, fc_PP, fc_phs1, fc_ptp1, fc_lm):
        # sourcery skip: dict-literal, inline-immediately-returned-variable, merge-dict-assign
        postData = dict()
        postData['date'] = date
        postData['swh'] = fc_hs
        postData['mwd'] = fc_dir
        postData['mwp'] = fc_t01
        postData['pp1d'] = fc_PP
        postData['shts'] = fc_phs1
        postData['mpts'] = fc_ptp1
        postData['mwl'] = fc_lm
        return postData

    def grttxt(self, _date):
        """
        构建预报单信息
        :param _date: 预报单日期
        :return: 0-生成预报数据成功  大于等于1-生成预报数据失败
        """
        isTemp = False  # 是否未来日期的扩展任务
        today = strDateTime.getToday()
        isTemp = _date <= today
        file_path = self._output_txt
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        # 写入文件内容----------------------------------begin
        _date0 = _date
        list_hours = [m_hours]
        # 写入文件头----------------------------------begin
        fh_time = "time".rjust(10, " ")  # 第一列日期留空白
        fh_hs = "swh".rjust(10, " ")  # 有效波高
        fh_dir = "mwd".rjust(10, " ")  # 波向
        fh_t01 = "mwp".rjust(10, " ")  # 平均周期
        fh_lm = 'mwl'.rjust(10, " ")  # 波长
        fh_PP = "pp1d".rjust(10, " ")  # 谱峰周期
        fh_phs1 = 'shts'.rjust(10, " ")  # 涌浪有效波高
        fh_ptp1 = 'mpts'.rjust(10, " ")  # 涌浪谱峰周期

        for k in range(1):
            tmp = list_hours[k]
            file_name = self._name + strDateTime.addDays(_date, k) + ".txt"
            file = os.path.join(file_path, file_name)

            with open(file, "w") as fl:
                fl.writelines([fh_time, fh_hs, fh_dir, fh_t01, fh_lm, fh_PP, fh_phs1, fh_ptp1, "\n"])
            # 写入文件头----------------------------------end

                for i in range(max(tmp) + 1):  # +72

                    fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                    fc_hs, fc_t01, fc_dir, fc_PP, fc_phs1, fc_ptp1, fc_lm = self.getfore(_date0, i)  # 获得数据

                    if fc_PP.strip() == "--":  # 谱峰周期为空时，采用2.99代替
                        fc_PP = str(2.99).rjust(10, " ")
                    if i in m_hours:
                        fl.writelines([fc_time, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1, "\n"])
                        print(f"hour in={str(i)}")
                    else:
                        print(f"hour={str(i)}")

    def grtjson(self, _date):
        
        file_path = self._output_json
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        _date0 = _date
        list_hours = [m_hours]
        # 创建json数据----------------------------------begin
        for k in range(1):
            tmp = list_hours[k]
            list = []
            for i in range(max(tmp) + 1):

                fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                fc_hs, fc_t01, fc_dir, fc_PP, fc_phs1, fc_ptp1, fc_lm = self.getnum(_date0, i)  # 获取数据

                if np.isnan(fc_PP):  # 谱峰周期为空时，采用2.99代替
                    fc_PP = 2.99

                JsonData = self.getJsonData(fc_time, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1)  # dict()
                list.append(JsonData)
                # 创建json数据----------------------------------end
                # 创建json文件----------------------------------begin
                file_name = self._name + strDateTime.addDays(_date, k) + ".json"
                file = os.path.join(file_path, file_name)
                # new_dict = {"location": {"No": "rzlssh01", "name": "日照岚山港区原油码头", "lon": lon_g, "lat": lat_g}, "data": list}
                new_dict = {"location": self._new_dict, "data": list}
                with open(file, "w") as f:
                    json.dump(new_dict, f, ensure_ascii=False, indent=10)
                # 写入文件内容----------------------------------end



# -*- coding:utf-8 -*-
"""
GFS下载合并后的风场 ---> 插值
"""
import os
import datetime
import time
from datetime import timedelta
import xarray as xr
import numpy as np
import pandas as pd
import argparse
from netCDF4 import Dataset
from christmas.commonCode import make_dir
from christmas import cprintf, osprint
from christmas.read_conf import read_conf


def para_iss():  # sourcery skip: merge-duplicate-blocks, remove-redundant-if
    # 获得默认参数
    nd = datetime.datetime.now()  # 按当天时间运行
    now_day = nd.strftime('%Y%m%d')
    time_now = int(nd.strftime('%H%M'))
    if time_now <= 630:     # 2:30后12不在运行
        file_List = '12'    # [12, 18, 0, 6]
    elif time_now <= 1230:  # 8:30后12,18不在运行
        file_List = '18'    # [18, 0, 6]
    elif time_now <= 1830:  # 14:30后12,18,00不在运行
        file_List = '00'    # [0, 6]
    elif time_now <= 2230:  # 20:30后12,18,00,06不在运行
        file_List = '06'    # [6]
    else:
        file_List = '06'
    # 添加外参
    parser = argparse.ArgumentParser(description="-d 日期 ;-n 风场时次文件夹")
    parser.add_argument('-d', default=now_day, help='输入时间格式为:20200101', type=str)
    parser.add_argument('-n', default=file_List, help='输入次数格式为:12', type=str)

    # 获取外参
    args = parser.parse_args()
    if len(args.d) != 8:
        osprint('输入时间格式为:20200101')
        exit()
    if len(args.n) != 2:
        osprint('输入次数格式为:12')
        exit()
    return args.d, args.n


def find_file(file_dir, prefix):
    all_file = os.listdir(file_dir)
    file_name = []
    file_path = []
    for f in all_file:
        if f.startswith(prefix):
            file_name.append(f)
            file_path.append(os.path.join(file_dir, f))
    return file_path


def seperation(_date, _conf_file='Configures/Interp_gfs_wind.conf'):
    para_conf = read_conf(_conf_file)  # 读取配置文件
    output_path = para_conf['Out_Path']
    file_name = para_conf['Region_Folder']

    needday = _date
    input_dir = os.path.join(output_path, file_name, needday)
    file_list = find_file(input_dir, 'w')
    out_dir = os.path.join(output_path, file_name,  needday, needday)
    for f in file_list:
        if not os.path.exists(out_dir):
            os.makedirs(out_dir)

        data = Dataset(f)
        lat = data.variables['latitude'][:]
        lon = data.variables['longitude'][:]
        timeNC=data.variables['time'][:]
        u_new=data.variables["U10"][:]
        v_new=data.variables["V10"][:]
        value_new=data.variables["v_VM"][:]
        dir_new=data.variables["dir_VM"][:]

        for i in range(24):
            outname = f'{file_name}_' + f[-11:-3] + str(i).zfill(2) + '.nc'
            out=os.path.join(out_dir, outname)
            da = Dataset(out, "w", format="NETCDF3_64BIT")

            da.createDimension("longitude", len(lat))
            da.createDimension("latitude", len(lon))
            da.createDimension("time", 1)

            da.createVariable("longitude", np.float64, "longitude")
            da.createVariable("latitude", np.float64, "latitude")
            da.createVariable("time", "f", "time")
            da.createVariable("U10", "f", ("time", "latitude", "longitude"))
            da.createVariable("V10", "f", ("time", "latitude", "longitude"))
            da.createVariable("v_VM", "f", ("time", "latitude", "longitude"))
            da.createVariable("dir_VM", "f", ("time", "latitude", "longitude"))

            da.variables["longitude"][:] = lon[:]
            da.variables["latitude"][:] = lat[:]
            da.variables["time"][:] = timeNC[i]
            da.variables["U10"][:] = u_new[i, :, :]
            da.variables["V10"][:] = v_new[i, :, :]
            da.variables["v_VM"][:] = value_new[i, :, :]
            da.variables["dir_VM"][:] = dir_new[i, :, :]

            da.close()


def run_interp2d(_date, _num, _conf_file='Configures/Interp_gfs_wind.conf'):
    # py_file_path = os.path.dirname(os.path.abspath(__file__))  # 当前python文件所在位置

    para_conf = read_conf(_conf_file)  # 读取配置文件
    lonmin = para_conf['Lon_min']  # 经度最小值
    lonmax = para_conf['Lon_max']  # 经度最大值
    latmin = para_conf['Lat_min']  # 纬度最小值
    latmax = para_conf['Lat_max']  # 纬度最大值
    Res_point = para_conf['Res_point']  # 格点数量

    num_grid = int(round(latmax - latmin, 2) * Res_point)   # 创建插值格点
    lon_new = np.linspace(lonmin, lonmax, num_grid)
    lat_new = np.linspace(latmin, latmax, num_grid)

    gfs_path = para_conf['Gfs_Path']                          # gfs文件夹路径
    Out_Path = para_conf['Out_Path']                          # 输出文件夹路径
    Region_Folder = para_conf['Region_Folder']                # 小区域文件夹路径
    output_path = os.path.join(Out_Path, Region_Folder, _date)
    # 输出文件夹路径
    make_dir(output_path)

    # 检测wind_3.nc文件（00:00检测文件夹12 00:06检测18）
    while True:
        if os.path.exists(f'{gfs_path}/{_date}/{_num}/wind_3.nc'):
            cprintf('INFO', f'{_date}的{_num}的wind_3文件存在')
            time.sleep(1)
            break
        else:
            cprintf('ERROR', f'未检测到{_date}的{_num}的wind_3文件,等待10分钟')
            time.sleep(10*60)

    # 合并前一天和当天的wind_3.nc文件
    nd = datetime.datetime.strptime(_date, '%Y%m%d')
    ago_date = (nd + timedelta(days=-1)).strftime('%Y%m%d')
    nc_data1 = xr.open_dataset(f'{gfs_path}/{ago_date}/{_num}/wind_3.nc')
    nc_data2 = xr.open_dataset(f'{gfs_path}/{_date}/{_num}/wind_3.nc')
    nc_data = xr.concat([nc_data2, nc_data1], dim='time')
    nc_data1.close()
    nc_data2.close()
    # 去重
    _, index = np.unique(nc_data['time'], return_index=True)
    nc_data = nc_data.isel(time=index)

    # 时区转换
    nc_data['time'] = nc_data['time'] + pd.Timedelta('8 hours')
    # 更改变量名字
    nc_data = nc_data.rename({'UGRD_10maboveground': 'U10', 'VGRD_10maboveground': 'V10'})
    # 插值到小区域并按时间截取
    for i in range(9):  # 天数(9天)
        cprintf('SUCCESS', f'正在处理{(nd + timedelta(days=i)).strftime("%Y%m%d")}')
        date_range = pd.date_range(start=nd+timedelta(days=i), end=nd+timedelta(days=i)+timedelta(hours=23), freq='H')
        nc_tem = nc_data.interp(longitude=lon_new, latitude=lat_new, time=date_range, method='linear')  # 时间空间插值
        # 添加新变量
        value = np.sqrt(nc_tem['U10'] ** 2 + nc_tem['V10'] ** 2)
        wind_dir = (np.arctan2(nc_tem['U10'], nc_tem['V10']) / np.pi) * 180 + 180
        nc_tem = nc_tem.update({'v_VM': value, 'dir_VM': wind_dir})
        # 压缩等级
        nc_tem.encoding['format'] = 'NETCDF4'
        nc_tem.encoding['engine'] = 'h5netcdf'
        nc_tem.encoding['dtype'] = 'float32'
        nc_tem['U10'].encoding['zlib'] = True
        nc_tem['U10'].encoding['complevel'] = 4
        nc_tem['V10'].encoding['zlib'] = True
        nc_tem['V10'].encoding['complevel'] = 4
        nc_tem['v_VM'].encoding['zlib'] = True
        nc_tem['v_VM'].encoding['complevel'] = 4
        nc_tem['dir_VM'].encoding['zlib'] = True
        nc_tem['dir_VM'].encoding['complevel'] = 4
        # 保存文件
        fout = f'{output_path}/{Region_Folder}_{(nd + timedelta(days=i)).strftime("%Y%m%d")}.nc'
        nc_tem.to_netcdf(fout)
    nc_data.close()

    seperation(_date, _conf_file)


if __name__ == '__main__':
    date, num = para_iss()
    run_interp2d(date, num)



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/3/21 14:28
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import conda_pack

# Pack environment my_env into my_env.tar.gz
conda_pack.pack(name="my_env")

# Pack environment my_env into out_name.tar.gz
conda_pack.pack(name="my_env", output="out_name.tar.gz")

# Pack environment located at an explicit path into my_env.tar.gz
conda_pack.pack(prefix="/explicit/path/to/my_env")



# -*- coding: utf-8 -*-

# @File    : pic_draw.py
# @Date    : 2023-01-17
# @Author  : Dovelet
import sys
import os
import re
import math
from ftplib import FTP
import netCDF4
import numpy as np
from netCDF4 import Dataset
from scipy.interpolate import griddata
from itertools import chain
import datetime
from scipy.interpolate import interp2d

py_file_path = os.path.dirname(__file__)

class Chazhi():
    def __init__(self,area):
        self.area=area
        self.svdir = '/data/Postprocess/160/Data/NC'

    def setting(self,area):
        """
        设置参数
        sv 为保存路径和文件中的区域和分辨率的表达式
        lonmin,lonmax 为目标区域经度范围
        latmin,latmax 为目标区域纬度范围
        re_ra 为resolution ratio即网格分辨率
        """
        if area=='dl':
            self.sv = '121.7E_122.0E_38.8N_39.1N_400'

            self.directory = '/data/ForecastSystem/WW3_6.07/Output/ww3_dalian'  # 大连港数据路径
            self.id='id_dlg_400.nc'
            self.water='water_dlg_400.nc'

            self.lonmin = 121.87
            self.lonmax = 121.96
            self.latmin = 38.92
            self.latmax = 39.01
            self.re_ra = 400

        elif area=='ls':
            self.sv = '119.21E_119.52E_34.95N_35.26N_120'

            self.directory = '/data/Output_160/ww3_lanshan'  # 岚山港数据路径
            self.id = 'id_lsg_120.nc'
            self.water = 'water_lsg_120.nc'

            self.lonmin = 119.21
            self.lonmax = 119.52
            self.latmin = 34.95
            self.latmax = 35.26
            self.re_ra = 120

        elif area=='yt':
            self.sv = '121.03E_121.16E_37.70N_37.83N_120'

            self.directory = '/data/ForecastSystem/WW3_6.07/Output/ww3_yantai'  # 烟台港数据路径
            self.id = 'id_ytg_120.nc'
            self.water = 'water_ytg_120.nc'

            self.lonmin = 121.03
            self.lonmax = 121.16
            self.latmin = 37.70
            self.latmax = 37.83
            self.re_ra = 120

        elif area=='rz':
            self.sv = '119.42E_119.60E_35.23N_35.41N_120'

            self.directory = '/home/ocean/haibotai/model_dlg'  # 日照港数据路径
            self.id = 'id_rzg_120.nc'
            self.water = 'water_rzg_120.nc'

            self.lonmin = 119.42
            self.lonmax = 119.60
            self.latmin = 35.23
            self.latmax = 35.41
            self.re_ra = 120

    def bianjie(self,n):
        if n == 0:
            waters = Dataset(f'/home/ocean/Oceanmax/Data/input/Interpolation/{self.water}')
        if n == 1:
            waters = Dataset(f'/home/ocean/Oceanmax/Data/input/Interpolation/{self.water}')
        water_id = waters['id'][:]
        water_id = water_id.reshape(1, len(water_id) * 4)
        id = water_id.tolist()
        id_fin = id[0]
        a = list(dict.fromkeys(id_fin))
        return a

    def near_inter(self,x_new, y_new, values, switchs):
        """
        x_new建立的插值格点的纬度，y_new建立的插值格点的经度
        values散点数据
        switchs控制是小区还是大区，0为大区，1为小区
        """
        if switchs == 0:
            m = Dataset(f'/home/ocean/Oceanmax/Data/input/Interpolation/{self.id}')
            index = m.variables['id'][:]
            index = list(index)
        if switchs == 1:
            m = Dataset(f'/home/ocean/Oceanmax/Data/input/Interpolation/{self.id}')
            index = m.variables['id'][:]
            index = list(index)

        grid_i, grib_j = x_new.shape
        result = np.zeros([grid_i, grib_j], np.float64)
        lat_number = len(x_new)
        lon_number = len(y_new)
        # 边界识别
        a = self.bianjie(switchs)
        for ilat in range(lat_number):
            for ilon in range(lon_number):
                if ilat * lat_number + ilon not in a:  # 水深大于0是陆地
                    result[ilat, ilon] = np.nan
                    continue
                else:

                    result[ilat, ilon] = values[index[ilat * lat_number + ilon]]

        return result

    def mkdir(self,path):
        # 去除首位空格
        path = path.strip()
        # 去除尾部 \ 符号
        path = path.rstrip("\\")
        # 判断路径是否存在
        isExists = os.path.exists(path)
        # 判断结果
        if not isExists:
            # 如果不存在则创建目录
            # 创建目录操作函数
            os.makedirs(path)
            return True
        else:
            return False

    def select_folder(self,filelist, nowday):
        #### 筛选出存放数据的文件夹并排序
        fd = datetime.datetime.strptime(nowday, "%Y%m%d")
        eta = (fd - datetime.timedelta(days=1)).strftime("%Y%m%d")

        delnum = 0
        for i in range(len(filelist)):
            if filelist[delnum][0] != '2' or filelist[delnum] < eta:
                del filelist[delnum]
            else:
                delnum = delnum + 1
        filelist.sort(key=lambda x: int(x[:]))

        mylist = filelist

        print(mylist)
        return mylist

    def folder_filename(self,dir, nowday):
        directory = dir
        filelist = os.listdir(directory)
        file_list = self.select_folder(filelist, nowday)
        return file_list

    def hourNums(self,eta_temp):

        fd = datetime.datetime.strptime(eta_temp, "%Y%m%d%H")
        eta = (fd + datetime.timedelta(hours=8)).strftime("%Y%m%d%H")

        return eta


    def hour(self,eta_temp):

        fd = datetime.datetime.strptime(eta_temp, "%H")
        eta = (fd + datetime.timedelta(hours=8)).strftime("%H")

        return eta


    def ncrename(self,outpath):
        filelist=os.listdir(outpath)
        filelist = sorted(filelist, key=lambda x: os.path.getmtime(os.path.join(outpath, x)))
        for i in range(0,16):
            filepath=os.path.join(outpath,filelist[i])
            os.system('rm -rf '+filepath)

        for i in range(-8,0):
            filepath = os.path.join(outpath, filelist[i])
            os.system('rm -rf ' + filepath)

        filelist = os.listdir(outpath)
        filelist = sorted(filelist, key=lambda x: os.path.getmtime(os.path.join(outpath, x)))

        for f in filelist:
            oriname=os.path.join(outpath, f)
            oritime=f[-13:-3]
            newtime=self.hourNums(oritime)
            f1=f.replace(oritime,newtime+'@')
            newname=os.path.join(outpath, f1)
            os.rename(oriname,newname)

        filelist = os.listdir(outpath)
        filelist = sorted(filelist, key=lambda x: os.path.getmtime(os.path.join(outpath, x)))

        for f in filelist:
            oriname = os.path.join(outpath, f)
            f1 = f.replace('@','')
            newname = os.path.join(outpath, f1)
            os.rename(oriname, newname)
        print('Rename is done.')


    def jion_strs(self,strs):
        return ' '.join(strs)

    def grid(self,_date):

        nowday = ''
        if len(_date) != 8:
            nowday = str(int(datetime.datetime.now().strftime("%Y%m%d")))  # 运行日期，从运行当天开始
        elif len(_date) == 8:
            nowday = str(_date)
        print(nowday)

        py_file_path = os.path.dirname(os.path.abspath(__file__))
        self.setting(self.area) # 初始化输入输出文件位置，文件保存名

        num_grid = int(round(self.latmax - self.latmin, 2) * self.re_ra)
        lon_new = np.linspace(self.lonmin, self.lonmax, num_grid)  # 创建插值格点
        lat_new = np.linspace(self.latmin, self.latmax, num_grid)
        lat_num_grid = int(round(self.latmax - self.latmin, 2) *self.re_ra)
        lon_num_grid = int(round(self.lonmax - self.lonmin, 2) * self.re_ra)
        y_new, x_new = np.meshgrid(lon_new, lat_new)

        filelist_need = self.folder_filename(self.directory, nowday)
        for item in filelist_need:
            print(f'{self.area}正在读取数据')
            filename = self.directory + '/' + item + '/' + 'ww3.' + item + '.nc'

            nc_obj = Dataset(filename)  # 读取数据

            lon = nc_obj['longitude']
            lat = nc_obj['latitude']
            time = nc_obj['time']
            t02 = nc_obj['t02']  # N*24
            hs = nc_obj['hs']  # N*24
            dir = nc_obj['dir']  # N*24
            for i in range(len(time)):
                t = str(i)  # 参数小时
                print('运行：', item + '-' + t)
                H = t.zfill(2)

                t02_new = np.ones((lat_num_grid, lon_num_grid))
                hs_new = np.ones((lat_num_grid, lon_num_grid))
                u_new = np.ones((lat_num_grid, lon_num_grid))
                v_new = np.ones((lat_num_grid, lon_num_grid))
                dir_new = np.ones((lat_num_grid, lon_num_grid))


                # 插值
                # SWH要素插值
                hs_new[:, :] = self.near_inter(x_new, y_new, hs[i, :], 1)
                dir_new[:, :] =self.near_inter(x_new, y_new, dir[i, :], 1)
                u_new[:, :] = -1 * np.sin(np.deg2rad(dir_new))
                v_new[:, :] = -1 * np.cos(np.deg2rad(dir_new))



                t02_new[:, :] = self.near_inter(x_new, y_new, t02[i, :], 1)

                # 保存
                t = str(i)
                H = t.zfill(2)  # 参数小时

                ## 6.awp ##
                svpath = self.svdir + '/mwp/mwp_' + self.sv + '/' \
                         + nowday + '/' + nowday  # 保存路径
                self.mkdir(svpath)
                ncfile = netCDF4.Dataset(svpath + '/mwp_' + self.sv + '_' + item + H + '.nc', 'w',
                                         format='NETCDF3_64BIT')
                ncfile.createDimension('lon', lon_num_grid)
                ncfile.createDimension('lat', lat_num_grid)
                ncfile.createVariable('lon', np.float64, ('lon'))
                ncfile.createVariable('lat', np.float64, ('lat'))
                ncfile.createVariable('awp', np.float64, ('lat', 'lon'))
                ncfile.variables['lon'][:] = lon_new[:]
                ncfile.variables['lat'][:] = lat_new[:]
                ncfile.variables['awp'][:, :] = t02_new[:, :]
                ncfile.close()
                ## 7.swh ##
                svpath = self.svdir + '/swh/swh_' + self.sv + '/' \
                         + nowday + '/' + nowday  # 保存路径
                self.mkdir(svpath)
                ncfile = netCDF4.Dataset(svpath + '/swh_' + self.sv + '_' + item + H + '.nc', 'w',
                                         format='NETCDF3_64BIT')
                ncfile.createDimension('lon', lon_num_grid)
                ncfile.createDimension('lat', lat_num_grid)
                ncfile.createVariable('lon', np.float64, ('lon'))
                ncfile.createVariable('lat', np.float64, ('lat'))
                ncfile.createVariable('dir', np.float64, ('lat', 'lon'))
                ncfile.createVariable('hs', np.float64, ('lat', 'lon'))
                ncfile.createVariable('u', np.float64, ('lat', 'lon'))
                ncfile.createVariable('v', np.float64, ('lat', 'lon'))
                ncfile.variables['lon'][:] = lon_new[:]
                ncfile.variables['lat'][:] = lat_new[:]
                ncfile.variables['hs'][:, :] = hs_new[:, :]
                ncfile.variables['u'][:, :] = u_new[:, :]
                ncfile.variables['v'][:, :] = v_new[:, :]
                ncfile.variables['dir'][:, :] = dir_new[:, :]
                ncfile.close()

        self.ncrename(self.svdir + '/mwp/mwp_' + self.sv + '/' \
                         + nowday + '/' + nowday)
        self.ncrename(self.svdir + '/swh/swh_' + self.sv + '/' \
                         + nowday + '/' + nowday)

if __name__ == '__main__':
    _date = sys.argv[-1]
    dl=Chazhi('dl')
    dl.grid(_date)

    ls=Chazhi('ls')
    ls.grid(_date)
    yt=Chazhi('yt')
    yt.grid(_date)



#!/home/ocean/.Software/Python-env/base/bin/python
# -*- coding: utf-8 -*-
#  日期 : 2023/3/1 20:22
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import os
import sys
from christmas.read_conf import read_conf
from baysalt.Single_Forecast_with_g2s import Grid2Sca, get_parser, print_args


def print_conf(_Conf_Grid2scatter):
    print('-----------------conf default-----------------')
    for key, value in _Conf_Grid2scatter.items():
        print(f'{key} : {value}')
    print('-----------------conf default-----------------')


if __name__ == '__main__':
    args, Conf_Grid2scatter_file = get_parser()
    if not args.conf:
        if os.path.basename(__file__) in sys.argv[-1]:
            print_args(args)
        X = Grid2Sca(args.lon,
                     args.lat,
                     _input='./output/',
                     value_list=args.iv,
                     title_list=args.ov,
                     _output=args.o,
                     _name=args.n,
                     _method=args.m
                     )
        X.makefore(_date=args.d, _json=False, _txt=True)
    else:
    
        if args.conf == '1':
            args.conf = Conf_Grid2scatter_file
    
        Conf_Grid2scatter = read_conf(args.conf)
        print_conf(Conf_Grid2scatter)
    #s_lon, s_lat, _input, _output, _name, value_list, title_list, m_hours=216, time_zone=8, search_point=4, _location=None, _method=None
        X = Grid2Sca(Conf_Grid2scatter['Lon_Scatter'],
                     Conf_Grid2scatter['Lat_Scatter'],
                     _input=Conf_Grid2scatter['InputDir'],
                     _output=Conf_Grid2scatter['OutputDir'],
                     _name=Conf_Grid2scatter['NamePrefix'],
                     value_list=Conf_Grid2scatter['ValueList'],
                     title_list=Conf_Grid2scatter['TitleList'],
                     m_hours=int(Conf_Grid2scatter['DurHours']),
                     time_zone=int(Conf_Grid2scatter['TimeZone']),
                     search_point=int(Conf_Grid2scatter['SearchPoint']),
                     _location=Conf_Grid2scatter['Location'],
                     _method=Conf_Grid2scatter['Method_griddata'])
        X.makefore(_date=int(Conf_Grid2scatter['Date_Scatter']), _json=False, _txt=True)
    



# -*- coding: utf-8 -*-

# @File    : new_cangnan.py
# @Date    : 2023-07-25
# @Author  : Dovelet
# -*- coding: utf-8 -*-

import netCDF4
import numpy as np
import os
import datetime
import baysalt.strDateTime as strDateTime
from scipy.interpolate import interp2d
from scipy.interpolate import interpn
import scipy.integrate
import json
import math

m_hours = list(range(216))
fvcom_dir='/data/Output_147/FVCOM_ECS_storm/Run/'

class Cangnan_Single:
    def __init__(self, _lon, _lat, _input, _output_txt, _output_json, _name, **_new_dict):
        self._lon = _lon
        self._lat = _lat
        self._input = _input
        self._output_txt = _output_txt
        self._output_json = _output_json
        self._name = _name
        self._new_dict = _new_dict
        self.p=None

    def getnum(self, _date, _idx):
        """
        获取ww3_djk数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
            '%Y%m%d') + '.nc'  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_Wp的nc文件，传入w中
        lons = w.variables['longitude'][:]
        lats = w.variables['latitude'][:]
        ww3hs = w.variables['hs'][idx, :, :]  # 有效波高
        ww3t02 = w.variables['t02'][idx, :, :]  # 平均周期
        ww3fp = w.variables['fp'][idx, :, :]  # 谱峰频率
        ww3dir = w.variables['dir'][idx, :, :]  # 波向
        ww3swellhs = w.variables['phs1'][idx, :, :]  # 涌浪有效波高
        ww3swelltm = w.variables['ptp1'][idx, :, :]  # 涌浪谱峰周期
        ww3lm = w.variables['lm'][idx, :, :]  # 波长

        fhs = interp2d(lons, lats, ww3hs, kind='linear')
        ft02 = interp2d(lons, lats, ww3t02, kind='linear')
        ffp = interp2d(lons, lats, ww3fp, kind='linear')
        fdir = interp2d(lons, lats, ww3dir, kind='linear')
        fswellhs = interp2d(lons, lats, ww3swellhs, kind='linear')
        fswelltm = interp2d(lons, lats, ww3swelltm, kind='linear')
        flm = interp2d(lons, lats, ww3lm, kind='linear')

        hs = fhs(self._lon, self._lat)
        # H4
        t02 = ft02(self._lon, self._lat)
        # H10
        mwd = fdir(self._lon, self._lat)
        # pp
        PP = 1 / ffp(self._lon, self._lat)
        # phs1
        phs1 = fswellhs(self._lon, self._lat)
        # ptp1
        ptp1 = fswelltm(self._lon, self._lat)
        # LM
        lm = flm(self._lon, self._lat)

        if phs1[0] >= 1e+10 or ptp1[0] >= 1e+10:  # 涌浪波高或谱峰周期为空时用一维谱计算
            ef_filename = self._input + date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime(
                '%Y%m%d') + '_ef.nc'  # ww3_ef nc文件名
            ww3_ef = netCDF4.Dataset(ef_filename)
            ww3ef = ww3_ef.variables['ef'][idx, :, :, :]
            f = ww3_ef.variables['f'][:]

            Z, Y, X = np.meshgrid(f, self._lat, self._lon, indexing='ij')
            ef = interpn((f, lats, lons), ww3ef, (Z, Y, X), method='linear')  # 维度和值的维度是正的

            # 涌浪要素
            # 新的涌浪计算公式
            sf1 = ef[:18, 0, 0]
            fs1 = f[:18]
            phs1[0] = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))  # 计算涌浪有效波高
            ptp1[0] = math.sqrt(
                scipy.integrate.trapz(sf1, fs1) / scipy.integrate.trapz(np.multiply(fs1 ** 2, sf1), fs1))  # 计算涌浪有效周期

        hs_rtn = np.round(hs[0], 2)
        t02_rtn = np.round(t02[0], 2)
        dir_rtn = np.round(mwd[0], 2)
        PP_rtn = np.round(PP[0], 2)
        phs1_rtn = np.round(phs1[0], 2)
        ptp1_rtn = np.round(ptp1[0], 2)
        lm_rtn = np.round(lm[0], 2)

        return hs_rtn, t02_rtn, dir_rtn, PP_rtn, phs1_rtn, ptp1_rtn, lm_rtn

    def getfore(self, _date, _idx):
        """
        获取ww3_djk数据,流场数据
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向
        """
        hs_rtn, t02_rtn, dir_rtn, PP_rtn, phs1_rtn, ptp1_rtn, lm_rtn = self.getnum(_date, _idx)

        fh_css,fh_csm,fh_csb,fh_cds,fh_cdm,fh_cdb=self.getcurrent(_date,_idx,self.p)
        return str(format(hs_rtn, '.2f')).rjust(12, " "), \
               str(format(t02_rtn, '.2f')).rjust(12, " "), \
               str(format(dir_rtn, '.2f')).rjust(12, " "), \
               str(format(PP_rtn, '.2f')).rjust(12, " "), \
               str(format(phs1_rtn, '.2f')).rjust(12, " "), \
               str(format(ptp1_rtn, '.2f')).rjust(12, " "), \
               str(format(lm_rtn, '.2f')).rjust(12, " "), \
                  str(format(fh_css, '.2f')).rjust(12, " "), \
                  str(format(fh_csm, '.2f')).rjust(12, " "), \
                    str(format(fh_csb, '.2f')).rjust(12, " "), \
                    str(format(fh_cds, '.2f')).rjust(12, " "), \
                    str(format(fh_cdm, '.2f')).rjust(12, " "), \
                    str(format(fh_cdb, '.2f')).rjust(12, " ")



    def getcurrent(self,_date,_idx,p):
        """
        获取fvcom数据
        :param _date: 日期
        :param _idx: 预报小时（需要做时间处理）
        :param p: fvcom网格点的索引
        :return: "currentspeeds","currentspeedm","currentspeedb","currentdirs","currentdirm","currentdirb"
        """
        # 时间处理，将UTC转为CST
        if _idx>=120:
            currentspeeds, currentspeedm, currentspeedb, currentdirs, currentdirm, currentdirb=np.nan,np.nan,np.nan,np.nan,np.nan,np.nan
            return currentspeeds, currentspeedm, currentspeedb, currentdirs, currentdirm, currentdirb
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        _idx = _idx - 8  # 向前8个钟头，找到CST当天的0点
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        if _idx<0:
            filename=os.path.join(fvcom_dir,date2.strftime('%Y%m%d')+'/output'+'/forecast_0001.nc')
        else:

            # 一个小天才的写法，哈哈哈哈
            filename=os.path.join(fvcom_dir,date2.strftime('%Y%m%d')+'/output'+'/forecast_0001.nc')
            if os.path.exists(filename)==False:
                filename=os.path.join(fvcom_dir,date1.strftime('%Y%m%d')+'/output'+'/forecast_'+str(int(_idx/24)+1).zfill(4)+'.nc')
        f_c = netCDF4.Dataset(filename) #fvcom nc
        u = f_c.variables['u'][:]
        v = f_c.variables['v'][:]
        currentspeeds = np.sqrt(u[idx,0, p] ** 2 + v[idx,0, p] ** 2)
        currentdirs= np.arctan2(v[idx,0, p], u[idx,0, p]) * 180 / np.pi
        currentspeedm = np.sqrt(u[idx,7, p] ** 2 + v[idx,7, p] ** 2)
        currentdirm = np.arctan2(v[idx,1, p], u[idx,1, p]) * 180 / np.pi
        currentspeedb = np.sqrt(u[idx,14, p] ** 2 + v[idx,14, p] ** 2)
        currentdirb = np.arctan2(v[idx,14, p], u[idx,14, p]) * 180 / np.pi

        currentspeeds=np.round(currentspeeds,2)
        currentspeedm=np.round(currentspeedm,2)
        currentspeedb=np.round(currentspeedb,2)
        currentdirs=np.round(currentdirs,2)
        currentdirm=np.round(currentdirm,2)
        currentdirb=np.round(currentdirb,2)

        return currentspeeds,currentspeedm,currentspeedb,currentdirs,currentdirm,currentdirb


    def get_current_sp(self,_date):
        """
        找到最近的fvcom数据点的索引
        :param _date: 日期。
        :return: 点位索引
        """
        _date = str(_date)
        filename = os.path.join(fvcom_dir, _date + '/output' + '/forecast_0001.nc')
        f_c = netCDF4.Dataset(filename)  # fvcom nc
        lonc= f_c.variables['lonc'][:]
        latc= f_c.variables['latc'][:]
        D=np.sqrt((lonc-self._lon)**2+(latc-self._lat)**2)
        idx=np.argmin(D)

        return idx

    @staticmethod
    def getTime(_date, _hour):
        """
        获取预报的日期/小时
        :param _date: 任务日期
        :param _hour: 小时
        :return: 日期/小时(10位长度的字符串)
        """
        if _hour == 215:
            _hour = 215

        date = strDateTime.convertToTime(_date)
        date_t = date + datetime.timedelta(hours=int(_hour))

        return date_t.strftime('%Y%m%d%H')

    @staticmethod
    def nan2str(_data):
        # 将nan转换为字符串
        if np.isnan(_data):
            return 'NaN'
        else:
            return _data

    @staticmethod
    def getJsonData(date, fc_hs, fc_dir, fc_t01, fc_PP, fc_phs1, fc_ptp1, fc_lm, fc_css, fc_csm, fc_csb, fc_cds, fc_cdm,fc_cdb):
        # sourcery skip: dict-literal, inline-immediately-returned-variable, merge-dict-assign
        postData = dict()
        postData['date'] = date
        postData['swh'] = fc_hs
        postData['mwd'] = fc_dir
        postData['mwp'] = fc_t01
        postData['pp1d'] = fc_PP
        postData['shts'] = fc_phs1
        postData['mpts'] = fc_ptp1
        postData['mwl'] = fc_lm
        postData['currentspeeds'] = Cangnan_Single.nan2str(fc_css)
        postData['currentspeedm'] = Cangnan_Single.nan2str(fc_csm)
        postData['currentspeedb'] = Cangnan_Single.nan2str(fc_csb)
        postData['currentdirs'] = Cangnan_Single.nan2str(fc_cds)
        postData['currentdirm'] = Cangnan_Single.nan2str(fc_cdm)
        postData['currentdirb'] = Cangnan_Single.nan2str(fc_cdb)
        return postData



    def grttxt(self, _date):
        """
        构建预报单信息
        :param _date: 预报单日期
        :return: 0-生成预报数据成功  大于等于1-生成预报数据失败
        """

        today = strDateTime.getToday()
        file_path = self._output_txt
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        # 写入文件内容----------------------------------begin
        _date0 = _date
        list_hours = [m_hours]
        # 写入文件头----------------------------------begin
        fh_time = "time".rjust(12, " ")  # 第一列日期留空白
        fh_hs = "swh".rjust(12, " ")  # 有效波高
        fh_dir = "mwd".rjust(12, " ")  # 波向
        fh_t01 = "mwp".rjust(12, " ")  # 平均周期
        fh_lm = 'mwl'.rjust(12, " ")  # 波长
        fh_PP = "pp1d".rjust(12, " ")  # 谱峰周期
        fh_phs1 = 'shts'.rjust(12, " ")  # 涌浪有效波高
        fh_ptp1 = 'mpts'.rjust(12, " ")  # 涌浪谱峰周期
        # 流场数据开头
        fh_css='currentspeeds'.rjust(12, " ")  # 表面流速
        fh_csm='currentspeedm'.rjust(12, " ")  # 中层流速
        fh_csb='currentspeedb'.rjust(12, " ")  # 底层流速
        fh_cds='currentdirs'.rjust(12, " ")  # 表面流向
        fh_cdm='currentdirm'.rjust(12, " ")  # 中层流向
        fh_cdb='currentdirb'.rjust(12, " ")  # 底层流向

        self.p = self.get_current_sp(_date)

        for k in range(1):
            tmp = list_hours[k]
            file_name = self._name + strDateTime.addDays(_date, k) + ".txt"
            file = os.path.join(file_path, file_name)

            with open(file, "w") as fl:
                fl.writelines([fh_time, fh_hs, fh_dir, fh_t01, fh_lm, fh_PP, fh_phs1, fh_ptp1,fh_css,fh_csm,fh_csb,fh_cds,fh_cdm,fh_cdb,"\n"])
                # 写入文件头----------------------------------end

                for i in range(max(tmp) + 1):  # +72

                    fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                    fc_hs, fc_t01, fc_dir, fc_PP, fc_phs1, fc_ptp1, fc_lm,fc_css,fc_csm,fc_csb,fc_cds,fc_cdm,fc_cdb = self.getfore(_date0, i)  # 获得数据

                    if fc_PP.strip() == "--":  # 谱峰周期为空时，采用2.99代替
                        fc_PP = str(2.99).rjust(12, " ")
                    if i in m_hours:
                        fl.writelines([fc_time, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1, fc_css,fc_csm,fc_csb,fc_cds,fc_cdm,fc_cdb,"\n"])
                        print(f"hour in={str(i)}")
                    else:
                        print(f"hour={str(i)}")

    def grtjson(self, _date):

        file_path = self._output_json
        if not os.path.exists(file_path):
            os.makedirs(file_path)

        _date0 = _date
        list_hours = [m_hours]
        self.p = self.get_current_sp(_date)
        # 创建json数据----------------------------------begin
        for k in range(1):
            tmp = list_hours[k]
            list = []
            for i in range(max(tmp) + 1):

                fc_time = self.getTime(_date0, i)  # 第一列日期留空白

                fc_hs, fc_t01, fc_dir, fc_PP, fc_phs1, fc_ptp1, fc_lm = self.getnum(_date0, i)  # 获取数据
                fc_css, fc_csm, fc_csb, fc_cds, fc_cdm, fc_cdb=self.getcurrent(_date0,i,self.p)

                if np.isnan(fc_PP):  # 谱峰周期为空时，采用2.99代替
                    fc_PP = 2.99

                JsonData = self.getJsonData(fc_time, fc_hs, fc_dir, fc_t01, fc_lm, fc_PP, fc_phs1, fc_ptp1,fc_css,fc_csm,fc_csb,fc_cds,fc_cdm,fc_cdb)  # dict()
                list.append(JsonData)
                # 创建json数据----------------------------------end
                # 创建json文件----------------------------------begin
                file_name = self._name + strDateTime.addDays(_date, k) + ".json"
                file = os.path.join(file_path, file_name)
                # new_dict = {"location": {"No": "rzlssh01", "name": "日照岚山港区原油码头", "lon": lon_g, "lat": lat_g}, "data": list}
                new_dict = {"location": self._new_dict, "data": list}
                with open(file, "w") as f:
                    json.dump(new_dict, f, ensure_ascii=False, indent=10)
                # 写入文件内容----------------------------------end


if __name__ == '__main__':
    input_path = '/data/ForecastSystem/WW3_6.07/Output/ww3_WPacific/'
    py_file_path = os.path.dirname(os.path.abspath(__file__))
    output_path_txt = os.path.join(py_file_path, './Data/single_txt/')
    output_path_json = os.path.join(py_file_path, './Data/single_json/')
    if not os.path.exists(output_path_txt):
        os.makedirs(output_path_txt)
    if not os.path.exists(output_path_json):
        os.makedirs(output_path_json)

    location_cnhd1 = {"No": "cnhd1", "name": "苍南核电1", "lon": 120.600, "lat": 27.260}
    name_cnhd1 = "cnhd1_120.521E_27.201N_"
    c=Cangnan_Single(120.600, 27.260, input_path, output_path_txt, output_path_json, name_cnhd1, **location_cnhd1)
    #c.grttxt('20230721')
    print('json')
    c.grtjson('20230721')
    print('end')



# -*- coding: utf-8 -*-
"""日期/时间相关的字符串处理

"""

import contextlib
# 日期：Rola 2016.12.23

import datetime


def getToday():
    """
    返回当天日期的 %Y%m%d 格式的8位字符串
    :return:
    """
    return datetime.datetime.now().strftime('%Y%m%d')


def getYesterday():
    """
    返回昨天日期的 %Y%m%d 格式的8位字符串
    :return:
    """
    today = datetime.datetime.now().strftime('%Y%m%d')

    return addDays(today, -1)


def getTomorrow():
    """
    返回明天日期的 %Y%m%d 格式的8位字符串
    :return:
    """
    today = datetime.datetime.now().strftime('%Y%m%d')

    return addDays(today, 1)


def getNow():
    """
    获取当前时刻的字符串(格式为 %Y-%m-%d %H:%M:%S)
    :return:
    """
    return datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')


def getNowstr(dmt_format='%Y%m%d%H%M%S'):
    """
    获取当前时刻的字符串(默认格式为 %Y%m%d%H%M%S)
    :return:
    """
    return datetime.datetime.now().strftime(dmt_format)


def convertToTime(strDate):
    """
    将 %Y%m%d 格式的8位字符串，转换为日期
    :param strDate: %Y%m%d 格式的8位日期字符串
    :return: datetime 类型的日期
    """
    date = datetime.datetime.now()  # 默认取当天日期

    with contextlib.suppress(Exception):
        if len(strDate) == 8:
            date = datetime.datetime.strptime(strDate, "%Y%m%d")
        elif len(strDate) == 10:
            date = datetime.datetime.strptime(strDate, "%Y%m%d%H")
        elif len(strDate) == 12:
            date = datetime.datetime.strptime(strDate, "%Y%m%d%H%M")
        elif len(strDate) == 14:
            date = datetime.datetime.strptime(strDate, "%Y%m%d%H%M%S")
    return date


def addDays(strDate, days):
    """
    在字符串日期上面增加天数
    :param strDate: %Y%m%d 格式的8位日期字符串
    :param days: 要增加的天数
    :return: 增加天数后的 %Y%m%d 格式的8位日期字符串
    """
    date = convertToTime(strDate)
    date_t = date + datetime.timedelta(days=int(days))

    return date_t.strftime('%Y%m%d')


def addTimes(strDate, days=0, hours=0, minutes=0, seconds=0):
    """
    在字符串日期上面增加时间(天/小时/分钟/秒)
    :param strDate: %Y%m%d 格式的8位日期字符串
    :param days: 要增加的天数
    :param hours: 要增加的小时数
    :param minutes: 要增加的分钟数
    :param seconds: 要增加的秒数
    :return: 增加时间间隔后的 等长度 字符串
    """
    date = convertToTime(strDate)
    date_t = date + datetime.timedelta(days=int(days), hours=int(hours), minutes=int(minutes), seconds=int(seconds))

    if len(strDate) == 8:
        return date_t.strftime('%Y%m%d')
    elif len(strDate) == 10:
        return date_t.strftime('%Y%m%d%H')
    elif len(strDate) == 12:
        return date_t.strftime('%Y%m%d%H%M')
    elif len(strDate) == 14:
        return date_t.strftime('%Y%m%d%H%M%S')


def addHours(strDate, hours):
    """
    在字符串日期上面增加小时数
    :param strDate: %Y-%m-%d %H:%M:%S 格式的8位日期字符串
    :param hours: 要增加的小时数
    :return: 增加天数后的 %Y%m%d 格式的8位日期字符串
    """
    date = convertToTime(strDate)
    date_t = date+datetime.timedelta(hours=int(hours))

    return date_t.strftime('%Y-%m-%d %H:%M:%S')


def getYear(strDate):
    # sourcery skip: inline-immediately-returned-variable, remove-redundant-slice-index
    """
    将 %Y%m%d 格式的8位字符串，获取其 YYYY 格式的年份
    :param strDate: %Y%m%d 格式的8位日期字符串
    :return: YYYY 格式的年份
    """
    year = strDate[0:4]

    return year


def getMonth(strDate):  # sourcery skip: inline-immediately-returned-variable
    """
    将 %Y%m%d 格式的8位字符串，获取其 MM 格式的月份
    :param strDate: %Y%m%d 格式的8位日期字符串
    :return: MM 格式的月份
    """
    month = strDate[4:6]

    return month


def getDay(strDate):  # sourcery skip: inline-immediately-returned-variable
    """
    将 %Y%m%d 格式的8位字符串，获取其 dd 格式的日期
    :param strDate: %Y%m%d 格式的8位日期字符串
    :return: dd 格式的日期
    """
    day = strDate[6:]

    return day


def compareDate(date1, date2):
    """
    比较两个日期的前后
    :param date1:
    :param date2:
    :return: 1 -- date1 早于 date2  0 --date1 与 date2 同一日期   -1 -- date1 晚于 date2
    """
    newDate1 = convertToTime(date1)
    newDate2 = convertToTime(date2)

    if newDate1 < newDate2:  # newDate1 早
        return 1
    elif newDate1 == newDate2:  # newDate1 等 newDate2
        return 0
    elif newDate1 > newDate2:  # newDate1 晚
        return -1


def addHoursByDate(strDate, hours):
    """
    在字符串日期上面增加小时数
    :param strDate:
    :param hours:
    :return:
    """
    date = convertToTime(strDate)
    date_t = date + datetime.timedelta(hours=int(hours))

    return date_t.strftime('%Y%m%d_%H%M%S')


def convertTimeToTime(strTime):
    """
    将 %Y%m%d_%H%M%S 格式的字符串，转换为日期时间
    :param strTime: %Y%m%d_%H%M%S 格式的时间字符串
    :return: datetime 类型的日期
    """

    with contextlib.suppress(Exception):
        date = datetime.datetime.strptime(strTime, "%Y%m%d_%H%M%S")
    return date


def getDDMMYYYY(strDate):
    """
    将日期字符串给为 DDMMYYYY 格式
    :param strDate:
    :return:
    """
    return f"{getDay(strDate)}-{getMonth(strDate)}-{getYear(strDate)}"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="baysalt",
    version="0.1.0.8",
    author="Christmas, Dovelet, Qidi Ma",
    author_email="273519355@qq.com",
    description="Some simple tools for baysalt",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'numpy',
        'netCDF4',
        'scipy',
        'baysalt_christmas',
    ],
)
# python3 setup.py sdist bdist_wheel
# twine upload dist/*



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#  日期 : 2023/4/11 20:22
#  作者 : Dovelet, Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
根据散点分辨率，最近插值最为接近实际情况
从散点数据提取单点数据， 并将单点数据转换为json｜txt文件
Have done!!!
"""
import argparse
import netCDF4
import numpy as np
import datetime
import os
import math
import scipy.integrate
import scipy.interpolate
import json
from christmas import makedirs
from christmas.cprintf import cprintf
import baysalt.strDateTime as strDateTime
from baysalt.Interp.Interp_g2s import print_args


class Sca2Sca:
    def __init__(self, s_lon, s_lat, _input, _output, _name, value_list, title_list, m_hours=216, time_zone=8, _location=None, _separate_tj=False):
        self.s_lon = s_lon  # 单点经度
        self.s_lat = s_lat  # 单点纬度
        self._input = _input  # 输入数据文件夹， 如：'/home/ocean/ForecastSystem/WW3_6.07/Output/ww3_dql/', 内存有'yyyymmdd/ww3.yyyymmdd.nc'文件
        self._output = _output  # 输出数据文件夹， 如：'/home/ocean/PostProcess/160/Data/single_json/'
        self._name = _name  # 输出文件名称， 如：'dql01_'
        self.value_list = value_list  # 需要提取的变量列表
        self.title_list = title_list  # 需要提取的变量名称列表
        self.m_hours = list(range(m_hours))  # 预报时效
        self.time_zone = time_zone  # 时间区间， 如：8
        self._location = _location  # json文件的location信息
        self._separate_tj = _separate_tj  # 是否区分txt和json文件夹
    
    def getsp(self, _date):
        # sourcery skip: inline-immediately-returned-variable
        """
        索引ww3点位,找到最近的ww3散点
        :param _date: 任务日期
        :return: 点位No
        """
        _date = str(_date)
        filename = os.path.join(self._input, f'{_date}/ww3.{_date}.nc')

        w = netCDF4.Dataset(filename)

        lon = w.variables['longitude'][:]
        lat = w.variables['latitude'][:]
        D = np.zeros((lon.size, 1))
        for i in range(lon.size):
            D[i] = (self.s_lon - lon[i]) ** 2 + (self.s_lat - lat[i]) ** 2  # 读取ww3_rizhao的nc文件，传入w中
        p = np.argwhere(D == min(D))

        return p
    
    def getData(self, _date, _idx, p):
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        t_idx = _idx - self.time_zone  # 向前time_zone个钟头，找到对应时区当天的0点, 如CST应为 - 8
        date2 = date1 + datetime.timedelta(hours=t_idx)
        idx = t_idx % 24
        filename = os.path.join(self._input, date2.strftime('%Y%m%d') + '/ww3.' + date2.strftime('%Y%m%d') + '.nc')  # ww3 nc文件名
        w = netCDF4.Dataset(filename)
        wind_swell = ["shww", "mpww", "hmaxw", "shts", "mpts", "hmaxs"]
        rely_on_hs = ["hmaxe", "h4", "h10"]
        current = ["currentspeeds", "currentspeedm", "currentspeedb", "currentdirs", "currentdirm", "currentdirb"]
        interp_value = []
        interp_num = []
        for ele in self.value_list:
            
            if ele in rely_on_hs:
                element = w.variables['hs'][idx, :][p[0][0]]
                temp = self.rely_on_hs(element, ele)
            elif ele in wind_swell:
                temp = self.wind_swell_cal(date2, idx, p, ele)
            elif ele in current:
                temp = self.gettpxo(_date, _idx, ele)
            elif ele in ["mdww", "mdts"]:
                temp = w.variables['dir'][idx, :][p[0][0]]
            elif ele == "pp1d":
                temp = 1 / w.variables['fp'][idx, :][p[0][0]]
                if np.isnan(temp):  # 谱峰周期为空时，采用2.99代替,过于不科学了嗷兄弟们
                    temp = 2.99
            else:
                temp = w.variables[ele][idx, :][p[0][0]]
            
            interp_value, interp_num = self.return_str_value(interp_value, interp_num, temp)
        return interp_value, interp_num
    
    @staticmethod
    def getTime(_date, _hour):
        """
        获取预报的日期/小时
        :param _date: 任务日期
        :param _hour: 小时
        :return: 日期/小时(10位长度的字符串)
        """
        date = strDateTime.convertToTime(_date)
        date_t = date + datetime.timedelta(hours=int(_hour))
        
        return date_t.strftime('%Y%m%d%H')
    
    @staticmethod
    def return_str_value(interp_value, interp_num, _value, retract=14):
        if str(_value).strip() == "--":
            _value = np.nan
        _value = np.round(_value, 2)
        interp_value.append(str(_value).format('.2f').rjust(retract, " "))
        interp_num.append(_value)
        return interp_value, interp_num
    
    @staticmethod
    def rely_on_hs(value, ele):
        if ele == "hmaxe":
            return np.round(1.46 * value, 2)
        elif ele == "h4":
            # 平均波高Hp
            Hp = (0.747 * value) / 1.129
            return np.round(2.024 * Hp, 2)
        elif ele == "h10":
            # 平均波高Hp
            Hp = (0.747 * value) / 1.129
            return np.round(1.712 * Hp, 2)
    
    def wind_swell_cal(self, _date, idx, p, ele):
        # sourcery skip: extract-duplicate-method
        """
        专门做风浪涌浪数据处理
        :param _date: _date是经过时区处理的
        :param idx: 时间点
        :param p: 点位索引
        :param ele: ele要素名
        :return: 要什么给什么
        """
        # 读取一位谱数据用于下面计算风浪和涌浪
        ef_filename = os.path.join(self._input, _date.strftime('%Y%m%d') + '/ww3.' + _date.strftime(
            '%Y%m%d') + '_ef.nc')
        ww3_ef = netCDF4.Dataset(ef_filename)
        ef = ww3_ef.variables['ef'][:, :, p[0][0]]
        f = ww3_ef.variables['f'][:]
        # 风浪要素计算准备
        sf2 = ef[idx, 18:30]
        fs2 = f[18:30]
        # 涌浪要素计算准备
        sf1 = ef[idx, 0:18]
        fs1 = f[:18]
        if ele == "shww":
            wndhs = 4 * math.sqrt(scipy.integrate.trapz(sf2, fs2))  # 计算风浪有效波高
            return np.round(wndhs, 2)
        # 新的风浪替代公式
        elif ele == "mpww":
            wndtm = math.sqrt(
                scipy.integrate.trapz(sf2, fs2) / scipy.integrate.trapz(np.multiply(fs2 ** 2, sf2), fs2))  # 计算风浪有效周期
            return np.round(wndtm, 2)
        elif ele == "hmaxw":
            wndhs = 4 * math.sqrt(scipy.integrate.trapz(sf2, fs2))
            wndmaxhs = 1.46 * wndhs
            return np.round(wndmaxhs, 2)  # 风浪最大波高
        elif ele == "shts":
            shs = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))  # 计算涌浪有效波高
            return np.round(shs, 2)
        elif ele == "mpts":
            swelltm = math.sqrt(
                scipy.integrate.trapz(sf1, fs1) / scipy.integrate.trapz(np.multiply(fs1 ** 2, sf1), fs1))  # 计算涌浪有效周期
            return np.round(swelltm, 2)
        elif ele == "hmaxs":
            shs = 4 * math.sqrt(scipy.integrate.trapz(sf1, fs1))
            maxshs = 1.46 * shs
            return np.round(maxshs, 2)  # 涌浪最大波高
        else:
            # 这句话是多余的，懒得改
            cprintf('ERROR', f"要素名有误：{ele},请检查风涌浪要素名称是否正确.")
    
    def gettpxo(self, _date, _idx, ele):
        # 时间处理，不需要转cst
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        date2 = date1 + datetime.timedelta(hours=_idx)
        filename = f'/data/Postprocess/160/Data/input/tpxo/{_date[:4]}_singlepoint_tpxo.nc'
        w = netCDF4.Dataset(filename, 'r', format='NETCDF4')
        lons = w.variables['longitude'][:]
        times = w.variables['TIME_CST'][:]
        
        row = next(
            (
                i
                for i in range(len(times))
                if ("".join(times[i].astype(str).tolist())) == date2.strftime('%Y%m%d%H')
            ),
            0,
        )
        col = np.argmin(np.abs(lons - self.s_lon))
        
        # print(row,col)
        u_m = w.variables['u'][row, col]  # 中层流速u
        v_m = w.variables['v'][row, col]  # 中层流速v
        # 计算流速和流向
        _dir = math.atan2(u_m, v_m)
        _dir = int(_dir * 180 / math.pi)
        if _dir < 0:
            _dir += 360
        speed = (u_m ** 2 + v_m ** 2) ** 0.5
        if ele[7:10] == "dir":
            return np.round(float(_dir), 2)
        elif ele == "currentspeeds":
            return np.round(float(1.2 * speed), 2)
        elif ele == "currentspeedm":
            return np.round(float(1.0 * speed), 2)
        else:
            return np.round(float(0.5 * speed), 2)
    
    def makefore(self, _date, _txt=True, _json=True):
        # sourcery skip: extract-duplicate-method, low-code-quality
        """
        txt 预报单的生成
        :param _date: 任务日期
        :param _txt: 是否生成txt文件
        :param _json: 是否生成json文件
        :return: none
        """
        if type(_date) == int:
            _date = str(_date)

        if self._separate_tj:
            file_path_txt = os.path.join(self._output, 'single_txt')
            file_path_json = os.path.join(self._output, 'single_json')
        else:
            file_path_txt = self._output
            file_path_json = self._output
        
        makedirs(file_path_txt, file_path_json)
        p = self.getsp(_date)  # 获取最近索引
        # 写入文件内容----------------------------------begin
        
        if _txt and not _json:  # 只生成txt文件
            cprintf('INFO', '生成txt文件中')
            
            tmp = self.m_hours
            
            file_name = self._name + _date + ".txt"
            
            file = os.path.join(file_path_txt, file_name)
            
            with open(file, "w") as fl:
                # 写入文件头----------------------------------begin
                fl.writelines("UTC{0:+03d}".format(self.time_zone).rjust(10, " "))
                for ele in self.title_list:
                    fl.writelines(ele.rjust(14, " "))
                fl.writelines('\n')
                # 写入文件头----------------------------------end
                
                for i in range(max(tmp) + 1):
                    fc_time = self.getTime(_date, i)  # 获取时间
                    txt_ele, json_ele = self.getData(_date, i, p)
                    del json_ele
                    if i in self.m_hours:
                        fl.writelines(fc_time)
                        fl.writelines(txt_ele)
                        fl.writelines('\n')
                        cprintf('SUCCESS', f"hour in={str(i)}")

        elif _txt:
            cprintf('INFO', '开始生成txt和json文件')

            tmp = self.m_hours
            json_data_list = []
            txt_data_list = []
            for i in range(max(tmp) + 1):
                # 创建json数据----------------------------------begin
                fc_time = self.getTime(_date, i)  # 第一列日期留空白
                
                txt_ele, json_ele = self.getData(_date, i, p)  # 获取数据
                del txt_ele
                for j in range(len(json_ele)):
                    if str(json_ele[j]).strip() == "--":
                        json_ele[j] = np.nan
                
                JsonData, TxtData = self.getTxtJsonData(fc_time, json_ele)  # dict()
                ## json
                json_data_list.append(JsonData)
                ## txt
                txt_data_list.append(TxtData)
                
                # 创建json数据----------------------------------end
                if i in self.m_hours:
                    cprintf('SUCCESS', f"hour in={str(i)}")
            # 创建json文件----------------------------------begin
            file_name_json = self._name + _date + ".json"
            file_json = os.path.join(file_path_json, file_name_json)
            if self._location is None:
                new_dict = {"location": {"lon": self.s_lon, "lat": self.s_lat}, "data": json_data_list}
            else:
                new_dict = {"location": self._location, "data": json_data_list}
            
            with open(file_json, "w") as f:
                json.dump(new_dict, f, ensure_ascii=False, indent=10)
            # 写入json文件内容----------------------------------end
            
            # 创建txt文件----------------------------------begin
            file_name_txt = self._name + _date + ".txt"
            file_txt = os.path.join(file_path_txt, file_name_txt)
            # 写入txt文件内容----------------------------------begin
            with open(file_txt, "w") as fl:
                fl.writelines("UTC{0:+03d}".format(self.time_zone).rjust(10, " "))
                for ele in self.title_list:
                    fl.writelines(ele.rjust(14, " "))
                fl.writelines('\n')
                for row in txt_data_list:
                    fl.writelines(row)
                    fl.writelines('\n')
            # 写入txt文件内容----------------------------------end
        elif _json:
            cprintf('INFO', '开始生成json文件')
            
            tmp = self.m_hours
            json_data_list = []
            for i in range(max(tmp) + 1):
                # 创建json数据----------------------------------begin
                fc_time = self.getTime(_date, i)  # 第一列日期留空白
                
                txt_ele, json_ele = self.getData(_date, i, p)  # 获取
                del txt_ele
                for j in range(len(json_ele)):
                    if str(json_ele[j]).strip() == "--":
                        json_ele[j] = np.nan
                
                JsonData = self.getJsonData(fc_time, json_ele)  # dict()
                ## json
                json_data_list.append(JsonData)
                
                # 创建json数据----------------------------------end
                if i in self.m_hours:
                    cprintf('SUCCESS', f"hour in={str(i)}")
            # 创建json文件----------------------------------begin
            file_name = self._name + _date + ".json"
            file = os.path.join(file_path_json, file_name)
            if self._location is None:
                new_dict = {"location": {"lon": self.s_lon, "lat": self.s_lat}, "data": json_data_list}
            else:
                new_dict = {"location": self._location, "data": json_data_list}

            with open(file, "w") as f:
                json.dump(new_dict, f, ensure_ascii=False, indent=10)
            # 写入json文件内容----------------------------------end
    
    def getJsonData(self, fc_time, json_ele):
        postData = {'date': fc_time}
        for i in range(len(self.value_list)):
            postData[self.title_list[i]] = np.round(np.float64(json_ele[i]), 2)
        
        return postData
    
    def getTxtJsonData(self, fc_time, json_ele):
        postJsonData = {'date': fc_time}
        PostTxtData = [fc_time]
        for i in range(len(self.value_list)):
            postJsonData[self.title_list[i]] = np.round(np.float64(json_ele[i]), 2)
            PostTxtData.append(str(json_ele[i]).format('.2f').rjust(14, " "))
        
        return postJsonData, PostTxtData


def get_parser():
    # Grid2scatter.conf文件路径
    baysalt_path = os.path.dirname(os.path.abspath(__file__))
    Conf_Grid2scatter_file = os.path.join(baysalt_path, 'Configures', 'Grid2scatter.conf')
    # 区分参数大小写
    parser = argparse.ArgumentParser(
        description=__doc__,
        epilog="End help",
        add_help=False,
        prog=f'./{os.path.basename(__file__)}',
        conflict_handler='error',
        formatter_class=argparse.RawTextHelpFormatter,
        allow_abbrev=True,
        prefix_chars='-+',
        # description='Usages',
    )
    parser.add_argument_group(
        'Usages Help', '%(prog)s \n'
                       '                -m liner \n'
                       '                -t 8 \n'
                       '                -p 4 \n'
                       '                -lon 114 \n'
                       '                -lat 35 \n'
                       '                -iv hs lm \n'
                       '                -ov swh mwl \n'
                       '                -i  /data/ForecastSystem/WW3_6.07/Output/ww3_WPacific/ \n'
                       '                -o  ./ \n'
                       '                -n try \n'
                       '                -d 20230301 \n'
                       '%(prog)s \n'
                       '                -conf 1 \n'
    )
    parser.usage = '%(prog)s [options] parameter'
    parser.add_argument('-h', '--help', action='help', help='show this help message and exit')
    parser.add_argument('-t', required=False, type=int, metavar='', default=8, help='set time zone')
    parser.add_argument('-lon', required=False, type=float, metavar='', default=0, help='set longitude')
    parser.add_argument('-lat', required=False, type=float, metavar='', default=0, help='set latitude')
    parser.add_argument('-iv', required=False, type=str, metavar='', default=['hs', 't02', 'dir', 'fp', 'lm', 'hmaxe'],
                        help='set input variable name', choices=['hs', 't02', 'dir', 'fp', 'lm', 'hmaxe'], nargs='+')
    parser.add_argument('-ov', required=False, type=str, metavar='',
                        default=['swh', 'mwp', 'mwd', 'pp1d', 'mwl', 'hmax'], help='set output variable name',
                        nargs='+')
    parser.add_argument('-i', required=False, type=str, metavar='',
                        default='/data/Output_160/ww3_djk/', help='set input path')
    parser.add_argument('-o', required=False, type=str, metavar='', default='./', help='set output path')
    parser.add_argument('-n', required=False, type=str, metavar='', default='s2s_', help='set output file name')
    parser.add_argument('-d', required=False, type=str, metavar='', default=strDateTime.getToday(), help='set date')
    parser.add_argument('-conf', required=False, type=str, metavar='', default=Conf_Grid2scatter_file, help='set conf')
    
    return parser.parse_args(), Conf_Grid2scatter_file


def example():

    args, _ = get_parser()
    print_args(args)

    X = Sca2Sca(119.78182518, 35.57300175, _input=args.i, _output=args.o, _name=args.n, value_list=args.iv, title_list=args.ov)
    X.makefore(_date=args.d, _json=True, _txt=True)


if __name__ == '__main__':
    example()
